@@ -1,3 +1,8 @@
+#ifdef __VMS
+#include <stdlib.h>
+#include <string.h>
+#endif
+
 #include "bigplus.h"
 
 
@@ -27,18 +32,23 @@ int main(int argc,char **argv)
     }
 
 
-    n1=new_num(100,0);
-    n2=new_num(100,0);
-    result=new_num(100,0);
-
     (void) strcpy(str,argv[1]);
     (void) strcpy(str2,argv[2]);
+
+    init_numbers();
+    init_num(&n1);
+    init_num(&n2);
+    init_num(&result);
+
     scale=0;
 
-    str2num(&n1,str,0);
-    str2num(&n2,str2,0);
-    bc_add (n1,n2,&result,0);
+    str2num(&n1,str,scale);
+    str2num(&n2,str2,scale);
+    bc_add (n1,n2,&result,scale);
     (void) fprintf(stderr,"%s\n",num2str(result));
+
+    free_num(&n1);
+    free_num(&n2);
     free_num(&result);
     exit(1);
 }
