@@ -3,7 +3,7 @@
  *
  *	RCS:
  *		$Revision: 1.1.1.1 $
- *		$Date: 2001/03/19 01:59:51 $
+ *		$Date: 2001/02/25 19:22:24 $
  *
  *	Security:
  *		Unclassified
@@ -282,19 +282,20 @@ typedef struct _SFontInfo
 ** prototypes/ declarations
 */
 
-Image *CreateBaseImage _FD ((unsigned int, unsigned int, int, int, int,
-        unsigned int));
-void FlattenImage _FD ((Image *, Image *, unsigned int, int, int));
-void ColormapSignature _FD ((Image *));
-Image *AllocateImageStruct _FD((void));
-void DestroyAnyImageStruct _FD ((Image **));
-int Read_Data _FD ((char *, int, int, FILE *));
-int ReadDataBlock _FD ((char *, FILE *));
-int LZWDecodeImage _FD ((Image *));
-void SyncImage _FD ((Image *));
-Image *DuplicateImage _FD ((Image *, unsigned int, unsigned int, unsigned int));
-int WriteGIFImage _FD ((Image *, char *));
-void LSBFirstWriteShort _FD ((unsigned int, FILE *));
+Image   *CreateBaseImage        _FD ((unsigned int, unsigned int, int, int, int,
+                                      unsigned int));
+void    FlattenImage            _FD ((Image *, Image *, unsigned int, 
+                                      int, int));
+void    ColormapSignature       _FD ((Image *));
+Image   *AllocateImageStruct    _FD((void));
+void    DestroyAnyImageStruct _FD ((Image **));
+int     Read_Data _FD ((char *, int, int, FILE *));
+int     ReadDataBlock _FD ((char *, FILE *));
+int     LZWDecodeImage _FD ((Image *));
+void    SyncImage _FD ((Image *));
+Image   *DuplicateImage _FD ((Image *, unsigned int, unsigned int, unsigned int));
+int     WriteGIFImage _FD ((Image *, char *));
+void    LSBFirstWriteShort _FD ((unsigned int, FILE *));
 unsigned int LZWEncodeImage _FD ((Image *, unsigned int));
 unsigned int UncompressImage _FD ((Image *));
 Image *ReadImage _FD ((char *));
@@ -321,4 +322,12 @@ int GetGIFsize _FD ((char *,unsigned int *,unsigned int *));
 Image *CropImage _FD ((Image *,RectangleInfo *));
 void commentImage _FD ((Image *,char *));
 
+#ifdef HAS_TRUE_TYPE 
+Image *ttfStringImage _FD ((char *str,char *font_file, int bgr,int bgg,int bgb,
+                int fgr,int fgg,int fgb, int border, int smooth,
+                int point_size, int x_res,int y_res,char *errstr));
+
+#endif /* HAS_TRUE_TYPE */
+
+
 #endif /* COMBINE_H */
