@@ -2,8 +2,8 @@
  *	FlattenImage()	-	flattens images
  *
  *	RCS:
- *		$Revision: 2.3 $
- *		$Date: 1996/05/03 02:21:34 $
+ *		$Revision: 1.1.1.1 $
+ *		$Date: 2001/02/25 19:22:24 $
  *
  *	Security:
  *		Unclassified
@@ -247,6 +247,36 @@ int
                     break;
                 }
 
+                case AnnotateCompositeOp:
+                {
+                    if (p->index == 0) /* Transparent */
+                    {
+                        red=q->red;
+                        green=q->green;
+                        blue=q->blue;
+                        index=q->index;
+                    }
+                    else if (p->index == MaxRGB) /* Opaque */
+                    {
+                        red=p->red;
+                        green=p->green;
+                        blue=p->blue;
+                        index=p->index;
+                    }
+                    else
+                    {
+                        red=(long) ((unsigned long)
+                         (p->red*p->index+q->red*(MaxRGB-p->index))/MaxRGB);
+                        green=(long) ((unsigned long)
+                         (p->green*p->index+q->green*(MaxRGB-p->index))/MaxRGB);
+                        blue=(long) ((unsigned long)
+                         (p->blue*p->index+q->blue*(MaxRGB-p->index))/MaxRGB);
+                        index=(long) ((unsigned long)
+                         (p->index*p->index+q->index*(MaxRGB-p->index))/MaxRGB);
+                    }
+
+                }
+
 	        }	/* end switch */
 
           /*
