@@ -67,8 +67,8 @@ create_gzip_tar ()
 
     mv /tmp/$arc_hist.$$ $pwd/$arc_hist
 
-    echo "running make pristine"
-    make pristine > /dev/null 2>&1
+    echo "running ./build --pristine"
+    ./build --pristine > /dev/null 2>&1
 #
 # go up
 #
