@@ -2,8 +2,8 @@
  *  Header file for Count
  *
  *  RCS:
-*      $Revision: 2.3 $
- *      $Date: 1996/05/03 02:20:22 $
+*      $Revision: 1.6 $
+ *      $Date: 2001/10/15 02:05:42 $
  *
  *  Security:
  *      Unclassified
@@ -75,7 +75,7 @@
 #include <sys/param.h>
 #endif
 
-#if HAVE_STDLIB_H 
+#if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 
@@ -109,6 +109,8 @@
 #endif
 #endif
 
+#include <mutils.h>
+
 #ifndef F_OK
 #define F_OK    0
 #endif
@@ -127,11 +129,11 @@
 #define SHOW_DATE       3
 #define SHOW_GIF_FILE   4
 #define SHOW_COUNTDOWN  5
-#define SHOW_VERSION    6 
+#define SHOW_VERSION    6
 
 #define ALIGN_LEFT          1
 #define ALIGN_CENTER        2
-#define ALIGN_RIGHT         3 
+#define ALIGN_RIGHT         3
 
 #define ALIGN_TOP_LEFT      4
 #define ALIGN_TOP_CENTER    5
@@ -154,6 +156,16 @@
 #define DATE_MMYYDD     5
 #define DATE_DDYYMM     6
 
+#ifdef __VMS
+# define DIR_SEPARATOR "" /* Already have ] at end of _DIR macros */
+/* VMS DIGIT_DIR ends in . so need to add subdirectory and ] - handled later */
+#define SYM_PREFIX "WWW_"
+#else
+#define DIR_SEPARATOR "/"
+#define SYM_PREFIX ""
+#endif
+
+
 #if __STDC__ || defined(sgi) || defined(_AIX)
 #define _Declare(formal_parameters) formal_parameters
 #else
@@ -171,13 +183,19 @@
 #define Extern
 #endif
 
-#define Version "2.5"
-#define Author          "ma_muquit@fccc.edu"
-#define Url             "http://www.fccc.edu/users/muquit/Count.html"
 
-#define IMAGE_COMMENT   "Count.cgi 2.5,(Jul-17-1999-1)\nBy Muhammad A Muquit\nhttp://www.fccc.edu/users/muquit/"
+#define Version "2.6"
+#define Author          "muquit@muquit.com"
+#define Url             "http://www.muquit.com/muquit/software/Count/Count.html"
 
+#define IMAGE_COMMENT   "@(#) Count.cgi 2.6-b10\nBy\nMuhammad A Muquit, http://www.muquit.com/"
+#define COUNT_COPYRIGHT "Copyright 1995-2001 Muhammad A Muquit"
+
+
+#ifdef MaxTextLength
+#undef MaxTextLength
 #define MaxTextLength 2048
+#endif /* Ultrix has MaxTextLength defined somewhere, -mm Sep-02-1999 */
 
 /*
 ** maximum number of allowable digits, should be enough for you?
@@ -199,7 +217,7 @@
 #define NoAutofcBlock           106
 #define NoStrictBlock           107
 #define NoRgbMappingBlock       108
-#define NO_RELOAD_BLOCK         109 
+#define NO_RELOAD_BLOCK         109
 
 #define NoLoginName             200
 #define NoDatafile              201
@@ -228,6 +246,7 @@ typedef struct _DigitInfo
         height;
 
     unsigned int
+        use_ttf,
         use_strip,
         comma;
 
@@ -261,6 +280,11 @@ typedef struct _DigitInfo
 
     char
         ddhead[100];
+
+
+	char
+		db_key[64]; /* gg => database key to access, used for displaying counts on a different page */
+
     char
         datafile[2048];
 
@@ -334,9 +358,13 @@ typedef struct _DigitInfo
         f_min,
         f_sec;
 
+
     int
         show_dhms;  /* show day hr min sec in countdown or not */
 
+	char
+		cd_fmt[2048]; /* gg => optional format for countdown display */
+
 
     /* alignment on the base image */
     int
@@ -348,12 +376,43 @@ typedef struct _DigitInfo
     int
         offset;     /* offset of counter image from base image in pixels */
 
+    /* TTF stuff */
+    char
+        zttfont_path[2048], /* abs path of the font file, will be created */
+        zttfont_name[64]; /* just the TrueType font name */
+
+    int
+        point_size;
+
+    int
+        border;
+
+    int
+        x_res;
+
+    int
+        y_res;
+
+    int
+        smooth; /* do anti alias or not */
+
+    unsigned char
+        bgr,
+        bgg,
+        bgb;
+
+    unsigned char
+        fgr,
+        fgg,
+        fgb;
+
+
 } DigitInfo;
 
 
 
 
-/* 
+/*
 ** singly linked list struct
 */
 typedef struct _Sllist
@@ -394,6 +453,10 @@ typedef struct _cfg
         count_reload;                       /* bool */
     int
         log_error_messages;                 /* bool */
+    int
+        log_visitor_info;                   /* bool */
+    int
+        log_rotation_interval;              /* seconds */
     int
         show_error_messages_to_browsers;    /* bool */
     Sllist
@@ -412,14 +475,14 @@ typedef struct _cfg
         Gdebug;
     Extern int
         GrgbMappingIsError;
-     
+
 #ifdef SYS_WIN32
   /*
   ** path of the virtual base of the CGI program
   */
    Extern char
         g_sZ_vbase[256];
-#endif  
+#endif
     Extern char
         g_sZ_ibase[256];
 
@@ -486,36 +549,36 @@ int     getcfgStrictMode        _Declare ((void));
 int     getcfgArgbDatabase      _Declare ((void));
 int     getcfgCountReload       _Declare ((void));
 int     getcfgLogErrmsg         _Declare ((void));
+int     getcfgLogVisitorInfo    _Declare ((void));
+int     getcfgLogRotationInterval  _Declare ((void));
 int     getcfgShowErrmsg        _Declare ((void));
+char    *getCurrentHits         _Declare((void));
 int         getExpireOffset     _Declare ((void));
 void        getHTTPexpires      _Declare ((int offset,char *expires,
                                            char *date,int length,
                                            int exp_size,int date_size));
-char        *getLocalTime       _Declare ((DigitInfo *di,int add_from));
+char        *getLocalTime       _Declare ((char *timezone_str,int add_from));
 int         getUserReload       _Declare ((void));
 void        GetRemoteReferer    _Declare ((char *, char *));
-int         GetLine             _Declare ((FILE *, char *));
-char        *GetTime            _Declare ((void));
+int         getLine             _Declare ((FILE *fp, char *buf,int buflen));
 int         hostAuthorized      _Declare ((char  *host));
 int         ignoreIP            _Declare ((char *remote_ip));
 void        initImageParams     _Declare ((DigitInfo *,FrameInfo *));
-int         isinname            _Declare ((char *string,char *mask));
 void        makeGIFPath         _Declare ((char *,int,char *,DigitInfo *));
 char        *mystrdup           _Declare ((char *));
-char        *mystristr          _Declare ((char *s,char *t));
-char        *mystrncat          _Declare ((char *dst,char *src,int n));
-char        *mystrncpy          _Declare ((char *dst,char *src,int n));
-char        *mystrtok           _Declare ((char *,char *));
-int         mystrcasecmp        _Declare ((char *,char *));
-int         mystrncasecmp       _Declare ((char *s1, char *s2, int n));
 void        MogrifyImage        _Declare ((Image **,DigitInfo *,FrameInfo *));
 time_t      netMktime           _Declare ((struct tm *));
 char        *rwCounterDataFile  _Declare ((char *filepath,
                                           char *remote_ip,int ignore_site,
                                           int use_st,int start_value,
                                           int do_increment));
+char        *rwCounterDatabase  _Declare ((char *dbpath, char *key,             /* gg => added dbkey */
+                                          char *remote_ip,int ignore_site,
+                                          int use_st,int start_value,
+                                          int do_increment));
+
 void        printList           _Declare ((Sllist *head));
-void        ParseConfig         _Declare ((void));
+void        readConfigFile      _Declare ((void));
 int         ParseQueryString    _Declare ((char *, DigitInfo *, FrameInfo *));
 int         ParseInteger        _Declare ((char *,int *,int,int));
 int         ParseDigitString    _Declare ((char *,char *,int));
@@ -529,14 +592,11 @@ int         ParseSize           _Declare ((char *,int *, int *));
 int         ParseError          _Declare ((char *,char *,int,char *));
 void        PrintHeader         _Declare ((void));
 void        releaseList         _Declare ((Sllist **head));
-char        *reverseString      _Declare ((char *));
 void        RemoveTrailingSp    _Declare ((char *));
-void        safeStrcpy          _Declare ((char *to,char *from,int length));
-void        safeStrcat          _Declare ((char *to_str,char *str,int length,
-                                           int ssc_size,int ssc_length));
 void        SendErrorImage      _Declare ((unsigned char *, int));
 void        setCache            _Declare ((int cache));
 void        setcfgSection       _Declare ((char **section));
+void        setCurrentHits      _Declare ((char **hits));
 void        setExpire           _Declare ((int exp));
 void        setUserReload       _Declare ((int i));
 void        SetLock             _Declare ((int));
@@ -544,8 +604,13 @@ void        StringImage         _Declare ((char *));
 void        UnsetLock           _Declare ((int));
 void        Warning             _Declare ((char *));
 void        makeCounterImage    _Declare ((DigitInfo *, FrameInfo *));
+void        doVisitorLog        _Declare ((void));
 
 void        multilineStringImage _Declare ((char *,int,int,int,int,int,int,int));
+int         grepForStringInDb   _Declare ((char *db_path,char *str,
+                                           char *errbuf));
+char        *grepForKeyInDb     _Declare ((char *db_path,char *key,
+                                           char *errbuf));
 
 
 #endif /* _COUNT_H */
