@@ -22,7 +22,7 @@ static Image *makeCountdownImage _Declare ((Image *,DigitInfo *));
 
 static char *ImagePrefix[]=
 {
-    "zero", "one", "two", "three", "four", "five", 
+    "zero", "one", "two", "three", "four", "five",
     "six", "seven", "eight", "nine",
 };
 
@@ -38,7 +38,7 @@ static char *ImagePrefix[]=
 **      none
 **
 **  Limitations and Comments:
-**      It writes the MIME header before the image as specified in 
+**      It writes the MIME header before the image as specified in
 **      CGI 1.1 spec. Before writing the header it also sends the headers
 **      to hint the browser to not cache the image.
 **
@@ -81,7 +81,7 @@ char
         Warning("Unable to create base image for string");
         return;
     }
-    
+
     font_info.do_bg=True;
     font_info.bgr=0;
     font_info.bgg=0;
@@ -104,7 +104,7 @@ char
     setExpire(0);
     PrintHeader();
     WriteGIFImage(image,(char *) NULL);
-    
+
 }
 /*
 ** mogrify image if needed
@@ -125,13 +125,13 @@ FrameInfo
     {
         if (digit_info->negate == True)
             NegateImage(*image);
-            
-        if (digit_info->Frame == True) 
+
+        if (digit_info->Frame == True)
         {
-            RGB 
+            RGB
                color,
                matte_color;
-            
+
             matte_color.red=frame_info->matte_color.red;
             matte_color.green=frame_info->matte_color.green;
             matte_color.blue=frame_info->matte_color.blue;
@@ -143,8 +143,8 @@ FrameInfo
             frame_info->y=frame_info->height;
             frame_info->width=(*image)->columns+(frame_info->width << 1);
             frame_info->height=(*image)->rows+(frame_info->height << 1);
-            frame_info->matte_color=matte_color; 
-        
+            frame_info->matte_color=matte_color;
+
             XModulate(&color,matte_color.red,matte_color.green,
                 matte_color.blue, HighlightModulate);
             frame_info->highlight_color.red=color.red;
@@ -164,9 +164,9 @@ FrameInfo
                 framed_image->class=DirectClass;
                 *image=framed_image;
             }
-        }    
+        }
         commentImage(*image,IMAGE_COMMENT);
-        
+
 #if 0
         (*image)->comments = (char *) malloc (1024*sizeof(char));
         if ((*image)->comments != (char *) NULL)
@@ -174,11 +174,11 @@ FrameInfo
             /*
             (void) sprintf((*image)->comments,"\n%s %s \n%s %s\n%s %s\n%s\n%s",
                 "Count.cgi", Version, "By", Author, "URL:", Url,
-                "Built:",GetTime());
+                "Built:",mutilsGetTime());
             */
             (void) strcpy((*image)->comments,IMAGE_COMMENT);
         }
-        
+
 #endif /* if 0 */
 
         if (digit_info->replace_color == True)
@@ -268,18 +268,7 @@ DigitInfo
     strip_image=(Image *) NULL;
 
 
-    /*
-    ** if comma is requesed, commaize (my new invented word!) the
-    ** digit. also it's meaningness to left pad while commizing,
-    ** therefore, don't left pad with zeros as well
-    */
-    if ((digit_info->display_type == (unsigned int) SHOW_COUNTER) &&
-        (digit_info->comma == True))
-    {
-        Commaize(digitbuf);
-        Debug2("after commaze=%s",digitbuf,0);
-    }
-    /*
+   /*
     ** if image strip is not used, we'll handle it as usual
     */
 if (digit_info->use_strip == False)
@@ -287,7 +276,7 @@ if (digit_info->use_strip == False)
     for (p=digitbuf; *p != '\0'; p++)
     {
         *tmpbuf='\0';
-        if(isdigit(*p))
+        if(isdigit((int) *p))
         {
             (void) sprintf(tmpbuf,GIF_PATH_FMT ".gif",
                 g_sZ_ibase,digit_info->ddhead,ImagePrefix[(int)*p-'0']);
@@ -306,7 +295,7 @@ if (digit_info->use_strip == False)
             if (*p == ',')
                 (void) sprintf(tmpbuf,GIF_PATH_FMT ".gif",
                     g_sZ_ibase,digit_info->ddhead,"comma");
-            
+
             if (*p == '-')
                 (void) sprintf(tmpbuf,GIF_PATH_FMT ".gif",
                     g_sZ_ibase,digit_info->ddhead,"dash");
@@ -343,10 +332,11 @@ else
     /*
     ** strip is in use. First of all, we will determine the with of the
     ** base image. The segments in the strip may be of varaible widths,
-    ** we'll handle it
+    ** we'll handle it. We won't have a overflow here.. ddhead is only 100
     */
     (void) sprintf(tmpbuf,GIF_PATH_FMT ".gif",
                           g_sZ_ibase,digit_info->ddhead,"strip");
+    Debug2("tmpbuf=%s",tmpbuf,0);
     strip_image=ReadImage(tmpbuf);
     if (strip_image == (Image *) NULL)
     {
@@ -368,11 +358,11 @@ else
         if (seg_array == (int *) NULL)
         {
             StringImage("Memory Allocation Failed for seg_array!");
-            exit(1);   
+            exit(1);
         }
         p=strip_image->comments;
         i=0;
-        while ((token=mystrtok(p,":")) != (char *) NULL)
+        while ((token=mutilsStrtok(p,":")) != (char *) NULL)
         {
             p=(char *) NULL;
             if (token != (char *) NULL)
@@ -396,7 +386,7 @@ else
 
         for (p=digitbuf; *p != '\0'; p++)
         {
-            if (isdigit(*p))
+            if (isdigit((int) *p))
             {
                 n=(*p-'0');
                 bwidth += (seg_array[n+2]-seg_array[n+1]);
@@ -481,7 +471,7 @@ if (digit_info->use_strip == False)
     for (p=digitbuf; *p != '\0'; p++)
     {
         *tmpbuf='\0';
-        if(isdigit(*p))
+        if(isdigit((int) *p))
             (void) sprintf(tmpbuf,GIF_PATH_FMT ".gif",
                 g_sZ_ibase,digit_info->ddhead,ImagePrefix[(int)*p-'0']);
         else
@@ -541,7 +531,7 @@ else
     for (p=digitbuf; *p != '\0'; p++)
     {
         *tmpbuf='\0';
-        if (isdigit(*p))
+        if (isdigit((int) *p))
         {
             n=(*p-'0');
             rinfo.x=seg_array[n+1];
@@ -573,12 +563,12 @@ else
                 rinfo.x=seg_array[14];
                 rinfo.width=seg_array[15]-seg_array[14];
             }
-            if (*p == '-')    
+            if (*p == '-')
             {
                 assertSegarray(16,max_idx,*p);
                 rinfo.x=seg_array[15];
                 rinfo.width=seg_array[16]-seg_array[15];
-            }    
+            }
             if (*p == 'D')
             {
                 assertSegarray(17,max_idx,*p);
@@ -610,14 +600,14 @@ else
         {
             FlattenImage(base_image,sub_image,ReplaceCompositeOp, base_width,0);
             base_width += sub_image->columns;
-            DestroyAnyImageStruct(&sub_image); 
+            DestroyAnyImageStruct(&sub_image);
         }
         else
         {
             StringImage("Failed to extract image from strip!");
             exit(1);
-        }    
-    } 
+        }
+    }
 }
     return(base_image);
 }
@@ -637,8 +627,8 @@ else
 **
 **  Limitations and Comments:
 **  0,0 is uppler left corener x is positive toward right and y is positive
-**  downward. 
-** 
+**  downward.
+**
 **  For ALIGN_TOP_LEFT, offset is from left and top edge of the base image.
 **  For ALIGN_TOP_CENTER, offset is from top edge of the base image
 **  For ALIGN_MID_LEFT, offset is from left edge of the base image.
@@ -731,7 +721,7 @@ Image
 
         case ALIGN_BOTTOM_RIGHT:
         {
-           digit_info->upl_x = base_image->columns - digit_image->columns 
+           digit_info->upl_x = base_image->columns - digit_image->columns
                                - offset;
            digit_info->upl_y = base_image->rows - digit_image->rows
                                 - offset;
@@ -754,6 +744,12 @@ FrameInfo
     int
         op;
 
+#ifdef HAS_TRUE_TYPE
+    char
+        zerrbuf[1024],
+        zfontpath[1024];
+#endif /* HAS_TRUE_TYPE */
+
     op=ReplaceCompositeOp;
     /* if GIF image, show it and get over with it. don't continue */
     if (digit_info->display_type == SHOW_GIF_FILE)
@@ -773,13 +769,51 @@ FrameInfo
         /* won't be here */
     }
 
-
-    /* combine digit images, may be countdown, counter or clock */
-    digit_image=CombineImages(digit_info->pdigits,digit_info);
-    if (digit_image == (Image *) NULL)
+    /*
+    ** if comma is requesed, commaize (my new invented word!) the
+    ** digit. also it's meaningness to left pad while commizing,
+    ** therefore, don't left pad with zeros as well
+    */
+    if ((digit_info->display_type == (unsigned int) SHOW_COUNTER) &&
+        (digit_info->comma == True))
     {
+        mutilsCommaize(digit_info->pdigits);
+    }
+
+
+#ifdef HAS_TRUE_TYPE
+
+    if (digit_info->use_ttf == True)
+    {
+        Debug2("TTF path=%s",digit_info->zttfont_path,0);
+        (void) strcpy(zfontpath,"/usr/local/fonts/TrueType/verdana.ttf");
+        digit_image=ttfStringImage(digit_info->pdigits,
+            digit_info->zttfont_path,
+            digit_info->bgr,digit_info->bgg,digit_info->bgb,
+            digit_info->fgr,digit_info->fgg,digit_info->fgb,
+            digit_info->border,
+            digit_info->smooth,
+            digit_info->point_size,
+            digit_info->x_res,digit_info->y_res,
+            zerrbuf);
+        if (digit_image == NULL)
+        {
+            Debug2("errbuf=%s",zerrbuf,0);
+            StringImage(zerrbuf);
+            exit(0);
+        }
+    }
+#endif   /* HAS_TRUE_TYPE */
+
+    if (digit_info->use_ttf == False)
+    {
+        /* combine digit images, may be countdown, counter or clock */
+        digit_image=CombineImages(digit_info->pdigits,digit_info);
+        if (digit_image == (Image *) NULL)
+        {
         StringImage("Failed! Check DigitDir in config.h or dd in QUERY_STRING");
         exit(0);
+        }
     }
 
     /* read gif image file if needed, we may be compositing */
@@ -814,7 +848,7 @@ FrameInfo
         */
         if (digit_info->upl_x < 0 && digit_info->upl_y < 0)
             updateCordinate(digit_info,gif_image,digit_image);
-           
+
         /*
         ** as we're combining with gif image, we'll handle transparency
         ** little differently. if trgb is specified we'll assume that
@@ -847,16 +881,16 @@ FrameInfo
                 }
             }
         }
- 
+
 
         if (countdown_image != (Image *) NULL)
         {
-            FlattenImage(gif_image,countdown_image,op, digit_info->upl_x, 
+            FlattenImage(gif_image,countdown_image,op, digit_info->upl_x,
                 digit_info->upl_y );
         }
         else
         {
-           FlattenImage(gif_image,digit_image,op,digit_info->upl_x, 
+           FlattenImage(gif_image,digit_image,op,digit_info->upl_x,
                 digit_info->upl_y );
         }
 
@@ -956,7 +990,7 @@ FrameInfo
             daytime=netMktime(ptm);
 #endif
             daytime +=diff;
-            ptm=localtime(&daytime);  
+            ptm=localtime(&daytime);
             Debug2("TIME GMT OFFSET=%s",asctime(ptm),0);
         }
         else
@@ -1052,7 +1086,7 @@ FrameInfo
             daytime=netMktime(ptm);
 #endif
             daytime +=diff;
-            ptm=localtime(&daytime);  
+            ptm=localtime(&daytime);
         }
         else
         {
@@ -1065,7 +1099,7 @@ FrameInfo
         */
         mon_dig1=(ptm->tm_mon-(ptm->tm_mon % 10)) / 10;
         mon_dig2=ptm->tm_mon % 10;
-        
+
         day_dig1=(ptm->tm_mday-(ptm->tm_mday % 10)) / 10;
         day_dig2=ptm->tm_mday % 10;
 
@@ -1084,7 +1118,7 @@ FrameInfo
         */
         if (show_full_year == 0)
         {
-            if (ptm->tm_year >= 100)    
+            if (ptm->tm_year >= 100)
             {
                 year_dig= 1900+ptm->tm_year;
                 show_full_year=1;
@@ -1165,7 +1199,7 @@ FrameInfo
                     break;
                 }
          }  /* switch */
-    
+
     di->pdigits=mystrdup(buf);
     makeCounterImage(di,fi);
 
@@ -1204,7 +1238,6 @@ int
     }
 }
 
-
 /*
 ** display the countdown
 ** dec-13-1998
@@ -1230,6 +1263,9 @@ FrameInfo
         now;
 
     int
+        y,m,d=0,        /* gg => used to calculate YMD display */
+        cdown_years,  /* gg => added for YMD display */
+        cdown_months, /* gg => added for YMD display */
         cdown_days,
         cdown_hours,
         cdown_minutes,
@@ -1238,15 +1274,8 @@ FrameInfo
     long
         diff;
 
-    target_tm.tm_year=digit_info->t_year - 1900;
-    target_tm.tm_mon=digit_info->t_mon - 1;
-    target_tm.tm_mday=digit_info->t_day;
-    target_tm.tm_hour=digit_info->t_hr;
-    target_tm.tm_min=digit_info->t_min;
-    target_tm.tm_sec=digit_info->t_sec;
-    target_tm.tm_isdst=(-1);
-
-
+    /* gg => target time calculations moved after t1 calculation */
+    /* gg => allows use of same local variables */
 
     /* if from values specified, calculate time_t for them -- mm,jul17,99 */
     if (digit_info->f_year >  0 &&
@@ -1270,6 +1299,15 @@ FrameInfo
     {
         /* local time */
         now=time(NULL);
+        /* gg => from_tm used for YMD calculation */
+        ptm_now=localtime(&now);
+        from_tm.tm_year=ptm_now->tm_year;
+        from_tm.tm_mon=ptm_now->tm_mon;
+        from_tm.tm_mday=ptm_now->tm_mday;
+        from_tm.tm_hour=ptm_now->tm_hour;
+        from_tm.tm_min=ptm_now->tm_min;
+        from_tm.tm_sec=ptm_now->tm_sec;
+        from_tm.tm_isdst=ptm_now->tm_isdst;
     }
 
     if (*digit_info->time_z != '\0')
@@ -1277,14 +1315,13 @@ FrameInfo
         /* offset from GMT in seconds */
         diff=checkTimezone(digit_info->time_z);
         now += diff;
-
         ptm_now=gmtime(&now);
         /*
-        ** Allow for UTC not being available - could make this specific to 
-        ** VMS < 7.0, but ANSI standard does allow gmtime to return NULL   
-        ** and the following line will not be executed unless it is NULL  
+        ** Allow for UTC not being available - could make this specific to
+        ** VMS < 7.0, but ANSI standard does allow gmtime to return NULL
+        ** and the following line will not be executed unless it is NULL
         */
-        if (! ptm_now) 
+        if (! ptm_now)
             ptm_now=localtime(&now);
 
 #ifdef HAVE_MKTIME
@@ -1296,13 +1333,70 @@ FrameInfo
     else
         t1=now;
 
+    /* gg => ********************************************************/
+    /* gg => added defaulting to current date/time for blank target */
+    /* if from values specified, calculate time_t for them -- mm,jul17,99 */
+    if (digit_info->t_year >  0 &&
+        digit_info->t_mon  >  0 &&
+        digit_info->t_day  >  0 &&
+        digit_info->t_hr   >= 0 &&
+        digit_info->t_min  >= 0 &&
+        digit_info->t_sec  >= 0)
+    {
+        target_tm.tm_year=digit_info->t_year - 1900;
+        target_tm.tm_mon=digit_info->t_mon - 1;
+        target_tm.tm_mday=digit_info->t_day;
+        target_tm.tm_hour=digit_info->t_hr;
+        target_tm.tm_min=digit_info->t_min;
+        target_tm.tm_sec=digit_info->t_sec;
+        target_tm.tm_isdst=(-1);
+
+        now=mktime(&target_tm);
+    }
+    else
+    {
+        /* local time */
+        now=time(NULL);
+        /* gg => target_tm used for YMD calculation */
+        ptm_now=localtime(&now);
+        target_tm.tm_year=ptm_now->tm_year;
+        target_tm.tm_mon=ptm_now->tm_mon;
+        target_tm.tm_mday=ptm_now->tm_mday;
+        target_tm.tm_hour=ptm_now->tm_hour;
+        target_tm.tm_min=ptm_now->tm_min;
+        target_tm.tm_sec=ptm_now->tm_sec;
+        target_tm.tm_isdst=ptm_now->tm_isdst;
+    }
+
+    if (*digit_info->time_z != '\0')
+    {
+        /* offset from GMT in seconds */
+        diff=checkTimezone(digit_info->time_z);
+        now += diff;
+        ptm_now=gmtime(&now);
+        /*
+        ** Allow for UTC not being available - could make this specific to
+        ** VMS < 7.0, but ANSI standard does allow gmtime to return NULL
+        ** and the following line will not be executed unless it is NULL
+        */
+        if (! ptm_now)
+            ptm_now=localtime(&now);
+
 #ifdef HAVE_MKTIME
-    t2=mktime(&target_tm);
+        t2=mktime(ptm_now);
 #else
-    t2=netMktime(&target_tm);
+        t2=netMktime(ptm_now);
 #endif
- 
+    }
+    else
+        t2=now;
+
+    /* gg => target date/time calculation modifications complete    */
+    /* gg => ********************************************************/
+
     tdiff=t2 - t1;
+    cdown_years=0; /* gg => default to unused */
+    cdown_months=0; /* gg => default to unused */
     cdown_days=tdiff/(60*60*24);
     cdown_hours=(tdiff % (60*60*24))/3600;
     cdown_minutes=(tdiff % 3600)/60;
@@ -1311,13 +1405,85 @@ FrameInfo
     pbuf=cMalloc(1024);
     if (digit_info->show_dhms == 1)
     {
-        (void) sprintf(pbuf,"%dD:%dH:%dM:%dS",
+        if (digit_info->use_ttf == True)
+        {
+            (void) sprintf(pbuf,"%d days %d hours %d minutes and %d seconds",
                    cdown_days,
                    cdown_hours,
                    cdown_minutes,
                    cdown_seconds);
-
+        }
+        else
+        {
+            (void) sprintf(pbuf,"%dD:%dH:%dM:%dS",
+                   cdown_days,
+                   cdown_hours,
+                   cdown_minutes,
+                   cdown_seconds);
+        }
     }
+    /***************************************************************/
+    /* gg => if difference is over a year, then display YMD format */
+    /* only used with TTF                                          */
+    else if (cdown_days > 365 && digit_info->use_ttf == True)
+    {
+        /* gg => determine years, months and days */
+        cdown_days=target_tm.tm_mday-from_tm.tm_mday;
+        if (cdown_days < 0 )
+        {
+            /* gg => calculating days in month before target_tm */
+            --cdown_months;
+            y = target_tm.tm_year + 1900;
+            m = target_tm.tm_mon-1;
+            if (m < 0)
+            {
+                --y;
+                m+=12;
+            }
+            switch (m) {
+                case 0: /* Jan */
+                case 2: /* Mar */
+                case 4: /* May */
+                case 6: /* Jul */
+                case 7: /* Aug */
+                case 9: /* Oct */
+                case 11: /* Dec */
+                    d=31;
+                    break;
+                case 3: /* Apr */
+                case 5: /* Jun */
+                case 8: /* Sep */
+                case 10: /* Nov */
+                    d=30;
+                    break;
+                case 1: /* Feb */
+                    if (y % 4 == 0)
+                    {
+                        d=29;
+                    }
+                    else
+                    {
+                        d=28;
+                    }
+                    break;
+            }
+            cdown_days+=d;
+        }
+        cdown_months+=target_tm.tm_mon-from_tm.tm_mon;
+        if (cdown_months < 0)
+        {
+            --cdown_years;
+            cdown_months+=12;
+        }
+        cdown_years+=target_tm.tm_year-from_tm.tm_year;
+
+            (void) sprintf(pbuf,digit_info->cd_fmt,
+                   cdown_years,
+                   cdown_months,
+                   cdown_days);
+    }
+    /* gg => YMD format complete                                   */
+    /***************************************************************/
     else
     {
         /* just show the days */
@@ -1331,10 +1497,10 @@ FrameInfo
 
 /*
 **  makeCountdownImage()
-**  if show from date is used. make a countdown image. 
+**  if show from date is used. make a countdown image.
 **
 **  Parameters:
-**  digit_image     
+**  digit_image
 **  digit_info
 **
 **  Return Values:
@@ -1395,7 +1561,7 @@ DigitInfo
     font_info.fgg=digit_info->pen_green;
     font_info.fgb=digit_info->pen_blue;
 
-    tstr=getLocalTime(digit_info,1);
+    tstr=getLocalTime(digit_info->time_z,1);
 
     string_width=strlen(tstr)*(gdFontSmall->w);
     if (digit_image->columns < string_width)
@@ -1487,7 +1653,7 @@ int
     /* extract the lines */
     lines=(char **) malloc(line_count*sizeof(char *));
     i=0;
-    while ((l=mystrtok(p,"\n")) != (char *) NULL)
+    while ((l=mutilsStrtok(p,"\n")) != (char *) NULL)
     {
         lines[i++]=mystrdup(l);
         p=(char *) NULL;
