@@ -2,15 +2,15 @@
  *  WWW document access counter
  *
  *  RCS:
- *      $Revision: 1.2 $
- *      $Date: 2001/04/08 16:15:51 $
+ *      $Revision: 1.5 $
+ *      $Date: 2001/10/14 16:46:15 $
  *
  *  Security:
  *      Unclassified
  *
  *  Description:
  *     main routine for WWW homepage access counter
- *  
+ *
  *  Input Parameters:
  *      type    identifier  description
  *
@@ -50,6 +50,10 @@
 #include "combine.h"
 #include "count.h"
 
+#ifdef USE_DATABASE
+#include <ldbm.h>
+#endif /* USE_DATABASE */
+
 #ifdef __VMS
 #include <fcntl.h>
 #include <unixio.h>
@@ -63,14 +67,9 @@
 #include "config.h"
 #endif
 
-#ifdef __VMS
-# define DIR_SEPARATOR "" /* Already have ] at end of _DIR macros */
-/* VMS DIGIT_DIR ends in . so need to add subdirectory and ] - handled later */
-#define SYM_PREFIX "WWW_"
-#else
-#define DIR_SEPARATOR "/"
-#define SYM_PREFIX ""
-#endif
+#ifndef SYS_WIN32
+#include "defs.h"
+#endif /* SYS_WIN32 */
 
 int main(argc,argv)
 int
@@ -94,23 +93,25 @@ char
         ignore_site=False;
 
 #ifdef SYS_WIN32
-	char
-		*script_name=NULL,
-	    drive[16];
+    char
+        *script_name=NULL,
+        drive[16];
 
 #endif /* SYS_WIN32 */
 
     char
+        **opp,
         hour[10],
         min[10],
         ampm[10],
         *s,
         *dbuf=(char *) NULL,
         *browser_type,
-        *penv,
         *remote_ip,
+        *penv,
         *query_string,
-        filename[MaxTextLength],
+        db_filename[MaxTextLength],
+        data_filename[MaxTextLength],
         tmpbuf[MaxTextLength],
         digitbuf[MaxTextLength];
 
@@ -120,6 +121,55 @@ char
     FrameInfo
         frame_info;
 
+    static char *options[]=
+    {
+"align     - align counter image on a base image. ",
+"bg and fg - Background and foreground color when TrueType fonts are used. ",
+"border    - Border space around counter when TrueType fonts are used. ",
+"chcolor   - change one color of the counter image. ",
+"cdt       - countdown target date. ",
+"cdf       - countdown from date. ",
+"cformat   - countdown YMD display format. ", /* gg => allow specifying countdown format */
+"comma     - show comma after 3rd digits from right. ",
+"dbkey     - database key to access. ", /* gg => allow viewing counts from different page */
+"dd        - digit directory. ",
+"degrees or angle - rotation angle of the image. ",
+"df        - counter data file if database support is not on. ",
+"dformat   - date format. ",
+"display   - counter, clock, date, countdown or image. ",
+"expires   - used to change image caching behavior of the browsers. ",
+"fd        - TrueType font directory name (not path). ",
+"font      - TrueType font filename (e.g. arial.ttf). ",
+"frgb      - Frame color. ",
+"ft        - Frame thickness ",
+"fy        - Full year (all digits). ",
+"image     - Display a GIF image. ",
+"incr      - Increment the counter or not. ",
+"istrip    - Use strip.gif or not. ",
+"lit       - Show literal string. ",
+"md        - Maximum number of digits to display. ",
+"negate    - Show the negative image of counter. ",
+"offset    - Offset from the counter image from the edge of the base image. ",
+"pad       - Left pad with zeros or not. ",
+"pt        - Point size of when TrueType fonts are used. ",
+"reload    - Count page reload or not. ",
+"rotate    - Rotate the counter image. ",
+"sdhms     - Show the day, hr, min, sec in countdown or not. ",
+"sfd       - Show \"from\" date while showing countdown. ",
+"sh        - Show the counter of keep it hidden. ",
+"smooth    - Smooth text or not when TrueType fonts are used. ",
+"srgb,prgb - Change source color to pen color. ",
+"st        - Start count number. ",
+"timezone  - Show time of that time zone. ",
+"tformat   - Time format, 12 or 24 hr. ",
+"tr        - Transparency On or Off. ",
+"trgb      - Transparency color. ",
+"xres      - Screen X resolution in dot/inch when TrueType fonts are used. ",
+"yres      - Screen Y resolution in dot/inch when TrueType fonts are used. ",
+"xy        - Composite the counter at x,y location of a base image. ",
+(char *) NULL
+    };
+
 /*
 **---------initialize globals------Starts---
 */
@@ -140,7 +190,8 @@ char
     *hour='\0';
     *min='\0';
     *ampm='\0';
-    *filename = '\0';
+    *db_filename='\0';
+    *data_filename = '\0';
     *digitbuf='\0';
     *tmpbuf = '\0';
     counter = 0;
@@ -151,6 +202,8 @@ char
     remote_ip=(char *) NULL;
 
 
+
+
     /*
     ** parse command line, this is only used for testing at commandline
     ** no command line flag is allowed in the web page while calling
@@ -159,15 +212,85 @@ char
 
     for (i=1; i < argc; i++)
     {
-        if (!strncmp(argv[i],"-debug",(int)strlen("-debug")))
+        s="-debug";
+        if(!strncmp(argv[i],s,(int) strlen(s)))
         {
             Gdebug=True;
             break;
         }
+
         s="-version";
         if(!strncmp(argv[i],s,(int) strlen(s)))
         {
-            (void) fprintf(stderr,"Count version %s\n",IMAGE_COMMENT);
+            printf ("%s\n",IMAGE_COMMENT);
+            printf ("%s\n",COUNT_COPYRIGHT);
+
+#ifdef USE_DATABASE
+            printf("\n*Database support is on.\n");
+            printf(" Compiled with ");
+#ifdef HAVE_BERKELEY_DB
+            printf("Berkeley DB\n");
+#ifdef DB_VERSION_MAJOR
+            printf("   Major version: %d\n",DB_VERSION_MAJOR);
+#endif /* DB_VERSION_MAJOR */
+#ifdef DB_VERSION_MINOR
+            printf("   Minor version: %d\n",DB_VERSION_MINOR);
+#endif /* DB_VERSION_MINOR */
+#endif /* HAVE_BERKELEY_DB */
+#ifdef LDBM_USE_DBBTREE
+            printf("Berkeyley DBTREE support\n\n");
+#endif /* LDBM_USE_DBBTREE */
+#ifdef LDBM_USE_DBHASH
+            printf("Berkeyley DBHASH support\n\n");
+#endif /* LDBM_USE_DBBTREE */
+
+#ifdef LDBM_USE_GDBM
+            printf("GNU dbm (gdbm) support\n");
+#endif /* LDBM_USE_GDBM */
+
+#ifdef LDBM_USE_SDBM
+            printf("builtin sdbm support\n");
+#endif /* LDBM_USE_SDBM */
+
+#ifdef LDBM_USE_NDBM
+            printf("ndbm support\n");
+#endif /* LDBM_USE_NDBM */
+
+#else /* ! USE_DATABASE */
+            printf("\n*No database support, df parameter must be used for counter\n");
+
+#endif /* USE_DATABSE */
+
+
+#ifdef HAS_TRUE_TYPE
+            printf("*TrueType font support is on\n");
+#else
+            printf("*No TrueType font support\n");
+#endif /* HAS_TRUE_TYPE */
+
+#ifdef COMPILE_OPTIONS
+            (void) fprintf(stderr,"\nCompile options:\n");
+            (void) fprintf(stderr,"%s\n",COMPILE_OPTIONS);
+#endif /* COMPILE_OPTIONS */
+
+#ifdef COMPILE_TIME
+            (void) fprintf(stderr,"Compiled on: %s\n",COMPILE_TIME);
+#endif /* COMPILE_TIME */
+
+#ifdef UNAME_HOST
+            (void) fprintf(stderr,"Compile time uname: %s\n",UNAME_HOST);
+#endif /* UNAME_HOST */
+
+            exit(1);
+        }
+
+        s="-options";
+        if(!strncmp(argv[i],s,(int) strlen(s)))
+        {
+            printf ("%s\n",IMAGE_COMMENT);
+            printf ("%s\n",COUNT_COPYRIGHT);
+            for(opp=options; *opp != (char *) NULL; opp++)
+                (void) printf("  %.1024s\n",*opp);
             exit(1);
         }
     }
@@ -187,9 +310,8 @@ char
     */
     _splitpath(script_name,drive,tmpbuf,NULL,NULL);
 
-    safeStrcpy(g_sZ_vbase,drive,sizeof(g_sZ_vbase)-1);
-    safeStrcat(g_sZ_vbase,tmpbuf,sizeof(g_sZ_vbase)-1,
-            sizeof(g_sZ_vbase),strlen(g_sZ_vbase));
+    mutilsSafeStrcpy(g_sZ_vbase,drive,sizeof(g_sZ_vbase)-1);
+    mutilsSafeStrcat(g_sZ_vbase,tmpbuf,sizeof(g_sZ_vbase)-1);
     /*
     ** note, g_sZ_vbase will have a trailing \
     ** example: cgi-bin\  in EMWAC
@@ -197,14 +319,14 @@ char
     ** c:\InetPub\scripts\ for MS IIS server
     */
 
-    safeStrcpy(g_sZ_ibase,g_sZ_vbase,sizeof(g_sZ_ibase)-1);
-    safeStrcat(g_sZ_ibase,DIGIT_DIR,sizeof(g_sZ_ibase)-1,
-            sizeof(g_sZ_ibase),strlen(g_sZ_ibase));
- 
+    mutilsSafeStrcpy(g_sZ_ibase,g_sZ_vbase,sizeof(g_sZ_ibase)-1);
+    mutilsSafeStrcat(g_sZ_ibase,DIGIT_DIR,sizeof(g_sZ_ibase)-1);
+
 #else
-    safeStrcpy(g_sZ_ibase,DIGIT_DIR,sizeof(g_sZ_ibase)-1);
-#endif  /* SYS_WIN32 */
-    
+    mutilsSafeStrcpy(g_sZ_ibase,DIGIT_DIR,sizeof(g_sZ_ibase)-1);
+    Debug2("g_sZ_ibase=%s",g_sZ_ibase,0);
+#endif  /* ! SYS_WIN32 */
+
 
     /* get the remote IP, we'll need it in many places */
 #ifdef __VMS
@@ -218,17 +340,17 @@ char
 #endif
 
     if (penv == (char *) NULL)
-        remote_ip="0.0.0.0";
+        remote_ip=mystrdup("0.0.0.0");
     else
         remote_ip=mystrdup(penv);
- 
-    Debug2("[%s] Count %s: --Debug--",GetTime(),Version);
+
+    Debug2("[%s] Count %s: --Debug--",mutilsGetTime(),Version);
 
     rc=CheckDirs();
     if (rc == 1)
     {
         *tmpbuf='\0';
-        safeStrcpy(tmpbuf,"CONFIG_DIR,DIGIT_DIR,DATA_DIR,LOG_DIR must differ",
+        mutilsSafeStrcpy(tmpbuf,"CONFIG_DIR,DIGIT_DIR,DATA_DIR,LOG_DIR must differ",
                 sizeof(tmpbuf)-1);
         Warning(tmpbuf);
 
@@ -236,20 +358,8 @@ char
         exit(1);
     }
 
-    /*
-    ** parse the config file
-    */
-    ParseConfig ();
-
-
-
-    /* print what we read, for debug only....... Remore asap */
-
-    /*
-    ** we will be only be here if everything is fine, otherwise we'll be
-    ** out from ParseConfig()
-    */
-
+    /* parse the config file and fill up s_cfg struct*/
+    readConfigFile();
 
 
     /*
@@ -259,10 +369,10 @@ char
     if (getcfgStrictMode() == True)
     {
         ihas_referer=authRemhost();
-    } 
+    }
     /* if not authorized, we won't be here */
 
- 
+
 
    /*
     ** get the user name from query string
@@ -273,15 +383,16 @@ char
     query_string = (char *) getenv("QUERY_STRING");
 #endif
 
+#ifndef USE_DATABASE
     if ((query_string == (char *) NULL) || (*query_string == '\0'))
     {
         *tmpbuf='\0';
-        safeStrcpy(tmpbuf,"Empty QUERY_STRING",sizeof(tmpbuf)-1);
+        mutilsSafeStrcpy(tmpbuf,"Empty QUERY_STRING",sizeof(tmpbuf)-1);
         Warning(tmpbuf);
         StringImage(tmpbuf);
         exit(1);
     }
-
+#endif /* ! USE_DATABASE */
     /*
     ** initialize to default values
     */
@@ -298,11 +409,9 @@ char
     }
 
     setUserReload(digit_info.user_reload);
-
 #if 0
-
     Debug2("\n",0,0);
-    Debug2("Parsed QUERY_STRING",0,0); 
+    Debug2("Parsed QUERY_STRING",0,0);
     Debug2(" ft=%d",frame_info.width,0);
     Debug2(" cache=%d", digit_info.cache,0);
     Debug2(" expires=%d",digit_info.expires,0);
@@ -323,7 +432,7 @@ char
         digit_info.pen_blue,
         0);
     Debug2(" maxdigits=%d",digit_info.maxdigits,0);
-    Debug2(" wxh [ignored,exists for compatibity] =%dx%d", 
+    Debug2(" wxh [ignored,exists for compatibity] =%dx%d",
         digit_info.width, digit_info.height);
     Debug2(" md=%d", digit_info.maxdigits,0);
     Debug2(" pad=%d", digit_info.leftpad,0);
@@ -345,24 +454,57 @@ char
     Debug2(" parsed QS--\n",0,0);
 #endif /* if 0 */
 
-
     /*
     ** now set some values for future retrieval
     */
+
     /*
     ** well, cache is not used anymore.. it's there just for backward
     ** compatibility. Only use "expires". Using cache and expires together
     ** creates too much confusion. If you to tell the browser not to cache
     ** specify expires=0, speicfy a value greater than 0 if want the browser
     ** to cache for that amount of seconds.
-    ** muquit@muquit.com Sun Mar 25 20:01:46 EST 2001 
+    ** muquit@muquit.com Sun Mar 25 20:01:46 EST 2001
     */
-/*
+    /*
     setCache(digit_info.cache);
-*/
+    */
     setExpire(digit_info.expires);
 
 
+    /* If using TrueType fonts, check the font has / in it */
+    if (digit_info.use_ttf == True)
+    {
+        if (checkfilename (digit_info.zttfont_name) != 0)
+        {
+            *tmpbuf='\0';
+            mutilsSafeStrcpy(tmpbuf,"/ found in TrueType font name, it's not legal",
+                    sizeof(tmpbuf)-1);
+            StringImage(tmpbuf);
+            exit(0);
+        }
+        /* create the font path and fill-up the member of SFontInfo struct */
+#ifdef SYS_WIN32
+        (void) strcpy(digit_info.zttfont_path,g_sZ_vbase);
+        (void) strcat(digit_info.zttfont_path,TT_FONT_DIR);
+        (void) strcat(digit_info.zttfont_path,DIR_SEPARATOR);
+        mutilsSafeStrcat(digit_info.zttfont_path,digit_info.zttfont_name,
+                sizeof(digit_info.zttfont_path)-1);
+
+#else
+
+        (void) strcpy(digit_info.zttfont_path,TT_FONT_DIR);
+        (void) strcat(digit_info.zttfont_path,DIR_SEPARATOR);
+        mutilsSafeStrcat(digit_info.zttfont_path,digit_info.zttfont_name,
+                sizeof(digit_info.zttfont_path)-1);
+#endif /* SYS_WIN32 */
+
+        Debug2("fontpath=%s",digit_info.zttfont_path,0);
+
+
+
+    }
+
     /*
     ** check if digit head has any path in it
     ** a bad bug, reported by Ojanpera <ojapa@japo.fi>
@@ -371,7 +513,7 @@ char
     if (checkfilename (digit_info.ddhead) != 0)
     {
         *tmpbuf='\0';
-        safeStrcpy(tmpbuf,"Illegal digit path!",sizeof(tmpbuf)-1);
+        mutilsSafeStrcpy(tmpbuf,"Illegal digit path!",sizeof(tmpbuf)-1);
         Warning(tmpbuf);
 
         StringImage(tmpbuf);
@@ -400,7 +542,7 @@ char
         if (checkfilename (digit_info.datafile) != 0)
         {
             *tmpbuf='\0';
-            safeStrcpy(tmpbuf,"Illegal datafile path!",sizeof(tmpbuf)-1);
+            mutilsSafeStrcpy(tmpbuf,"Illegal datafile path!",sizeof(tmpbuf)-1);
             Warning(tmpbuf);
 
             StringImage(tmpbuf);
@@ -452,7 +594,7 @@ char
             ** display a literal string. this will make sure that even if
             ** someone with a broken browser tries to access the counter
             ** remotely, will not mess up the datafile. of course we'll
-            ** miss some counts. because anyone accesses the page with a 
+            ** miss some counts. because anyone accesses the page with a
             ** broken browser like this will see a literal string
             */
             (void) strcpy(digit_info.literal,"888888");
@@ -496,11 +638,10 @@ char
                     if (remote_ip != (char *) NULL)
                     {
                         *tmpbuf='\0';
-                        safeStrcpy(tmpbuf,browser_type,sizeof(tmpbuf)-1);
-                        safeStrcpy(digitbuf,"browser without referer ",
+                        mutilsSafeStrcpy(tmpbuf,browser_type,sizeof(tmpbuf)-1);
+                        mutilsSafeStrcpy(digitbuf,"browser without referer ",
                                 sizeof(digitbuf)-1);
-                        safeStrcat(digitbuf,tmpbuf,sizeof(digitbuf)-1,
-                                sizeof(digitbuf),strlen(digitbuf));
+                        mutilsSafeStrcat(digitbuf,tmpbuf,sizeof(digitbuf)-1);
                         Warning(digitbuf);
                         *digitbuf='\0';
                     }
@@ -514,7 +655,7 @@ char
     */
     if (*digit_info.gif_file != '\0')
     {
-        
+
         /*
         ** security fix. oct-10-1997
         ** first check if the image file specified with image=
@@ -528,7 +669,7 @@ char
 
         makeGIFPath(tmpbuf,sizeof(tmpbuf),g_sZ_ibase,&digit_info);
         /* overwrite digit_info.gif_file */
-        safeStrcpy(digit_info.gif_file,tmpbuf,1023);
+        mutilsSafeStrcpy(digit_info.gif_file,tmpbuf,1023);
     }
 
     switch (display_what)
@@ -572,7 +713,7 @@ char
                 StringImage("No GIF file specified to display");
                 exit(1);
             }
-            /* 
+            /*
             ** check if counter is needed to be incremented
             */
 
@@ -583,7 +724,7 @@ char
                 /*
                 ** if datafile is RANDOM, just ignore counting
                 */
-                if (mystrcasecmp(digit_info.datafile,"random") == 0)
+                if (mutilsStrcasecmp(digit_info.datafile,"random") == 0)
                     digit_info.increment_counter=False;
                 /*
                 ** check increment. if true, we'll make the path of the
@@ -599,31 +740,65 @@ char
                         exit(0);
                     }
 #ifdef SYS_WIN32
-                    safeStrcpy(filename,g_sZ_vbase,sizeof(filename)-1);
-                    safeStrcat(filename,DATA_DIR,sizeof(filename)-1,
-                            sizeof(filename),strlen(filename));
-                    
-#else
-                    safeStrcpy(filename,DATA_DIR,sizeof(filename)-1);
-#endif
-                    safeStrcat(filename,DIR_SEPARATOR,sizeof(filename)-1,
-                            sizeof(filename),strlen(filename));
-                    safeStrcat(filename,digit_info.datafile,
-                            sizeof(filename)-1,sizeof(filename),
-                            strlen(filename));
+                    mutilsSafeStrcpy(data_filename,g_sZ_vbase,
+                            sizeof(data_filename)-1);
+                    mutilsSafeStrcat(data_filename,DATA_DIR,
+                            sizeof(data_filename)-1);
 
-                    Debug2("datafile path=%s",filename,0);
+#else
+                    mutilsSafeStrcpy(data_filename,DATA_DIR,
+                            sizeof(data_filename)-1);
+#endif
+                    mutilsSafeStrcat(data_filename,DIR_SEPARATOR,
+                            sizeof(data_filename)-1);
+                    mutilsSafeStrcat(data_filename,digit_info.datafile,
+                            sizeof(data_filename)-1);
+
+                    Debug2("datafile path=%s",data_filename,0);
                     if (remote_ip)
                         ignore_site=ignoreIP(remote_ip);
-                    /* 
+                    /*
                     ** NOTE: dbuf is malloc'd but we won't care to free it
                     ** as memory will be freed when the CGI exits
                     */
-                    dbuf=rwCounterDataFile(filename,
+#ifndef USE_DATABASE
+                    dbuf=rwCounterDataFile(data_filename,
                             remote_ip,ignore_site,False,
                             1,digit_info.increment_counter);
+#else /* use database */
 
-                }
+#ifdef SYS_WIN32
+                    /* make the full path of the database */
+                    (void) strcpy(db_filename,g_sZ_vbase);
+                    (void) strcat(db_filename,DATABASE_DIR);
+                    (void) strcat(db_filename,DIR_SEPARATOR);
+                    (void) strcat(db_filename,COUNT_DATABASE);
+#else
+                    /* make the full path of the database */
+                    (void) strcpy(db_filename,DATABASE_DIR);
+                    (void) strcat(db_filename,DIR_SEPARATOR);
+                    (void) strcat(db_filename,COUNT_DATABASE);
+#endif /* SYS_WIN32 */
+
+                    /* if df param is provided, act the old way */
+                    if (*digit_info.datafile != '\0')
+                    {
+                        Debug2("datafile path=%s",data_filename,0);
+                        dbuf=rwCounterDataFile(data_filename,
+                            remote_ip,ignore_site,False,
+                            1,digit_info.increment_counter);
+                    }
+                    else
+                    {
+                        Debug2(" XXXXX1 db=%s",db_filename,0);
+                        dbuf=rwCounterDatabase(db_filename,
+                            digit_info.db_key,                 /* gg => new parameter */
+                            remote_ip,ignore_site,False,
+                            1,digit_info.increment_counter);
+                    }
+
+#endif /* USE_DATABASE */
+                } /* digit_info.increment_counter */
 
                 /*
                 ** if literal string is specified, it might be later used
@@ -662,7 +837,7 @@ char
             if ((int) strlen(digit_info.literal) > 0)
             {
                 /* digitbuf buffer must be larger than MAX_DIGITS */
-                   safeStrcpy(digitbuf,digit_info.literal,MAX_DIGITS-1);
+                   mutilsSafeStrcpy(digitbuf,digit_info.literal,MAX_DIGITS-1);
 
                 /* need to look at it again --------------------*/
                 if (digit_info.comma == True)
@@ -670,7 +845,7 @@ char
                     /* make sure all are digits, if not turn off comma */
                     for (i=0; i < strlen(digitbuf); i++)
                     {
-                        if (!isdigit(digitbuf[i]))
+                        if (!isdigit((int) digitbuf[i]))
                         {
                             Debug2("non digit in lit, turning off comma",0,0);
                             digit_info.comma=False;
@@ -686,7 +861,7 @@ char
                     exit(0);
                  }
             }
-            else if (mystrcasecmp(digit_info.datafile,DF_FOR_RANDOM) == 0)
+            else if (mutilsStrcasecmp(digit_info.datafile,DF_FOR_RANDOM) == 0)
             {
                 srand(time(NULL));  /* Seed number generator */
                 counter = rand();   /* My psychic prediction of counter value */
@@ -699,17 +874,17 @@ char
                 *tmpbuf='\0';
                 (void) sprintf(tmpbuf,"%d",counter);
 
-                /* 
+                /*
                 ** I don't know why I did the following?
                 ** Nov-03-1997. muquit
                 */
                 /*
                 if (strlen(tmpbuf) > 10)
-                    safeStrcpy(digitbuf,tmpbuf,10);
+                    mutilsSafeStrcpy(digitbuf,tmpbuf,10);
                 else
                 */
 
-              safeStrcpy(digitbuf,tmpbuf,sizeof(digitbuf)-1);
+              mutilsSafeStrcpy(digitbuf,tmpbuf,sizeof(digitbuf)-1);
               digit_info.leftpad=False;
               dbuf=mystrdup(digitbuf);
               if (dbuf == (char *) NULL)
@@ -721,41 +896,44 @@ char
             else
             {
 #ifdef SYS_WIN32
-                    safeStrcpy(filename,g_sZ_vbase,sizeof(filename)-1);
-                    safeStrcat(filename,DATA_DIR,sizeof(filename)-1,
-                            sizeof(filename),strlen(filename));
+                    mutilsSafeStrcpy(data_filename,g_sZ_vbase,
+                            sizeof(data_filename)-1);
+                    mutilsSafeStrcat(data_filename,DATA_DIR,
+                            sizeof(data_filename)-1);
 #else
-                safeStrcpy(filename,DATA_DIR,sizeof(filename)-1);
+                    mutilsSafeStrcpy(data_filename,DATA_DIR,
+                            sizeof(data_filename)-1);
 #endif
-                safeStrcat(filename,DIR_SEPARATOR,sizeof(filename)-1,
-                        sizeof(filename),strlen(filename));
-                safeStrcat (filename,digit_info.datafile,sizeof(filename)-1,
-                        sizeof(filename),strlen(filename));
+                    mutilsSafeStrcat(data_filename,DIR_SEPARATOR,
+                            sizeof(data_filename)-1);
+                    mutilsSafeStrcat (data_filename,digit_info.datafile,
+                            sizeof(data_filename)-1);
 
                 /*
                 ** check if the counter file exists or not
                 */
                 if (getcfgAutoFileCreation() == False)
                 {
-                    if (CheckFile (filename) != 0)
+#ifndef USE_DATABASE
+                    if (CheckFile (data_filename) != 0)
                     {
-                        if (strlen(filename) > sizeof(tmpbuf)-60)
+                        if (strlen(data_filename) > sizeof(tmpbuf)-60)
                         {
-                            StringImage("buffer overflow in filename");
+                            StringImage("buffer overflow in data_filename");
                             exit(0);
                         }
                         *tmpbuf='\0';
                         (void) sprintf (tmpbuf,
-                    "Counter datafile \"%s\" must be created first!", filename);
+                    "Counter datafile \"%s\" must be created first!", data_filename);
                         Warning(tmpbuf);
                         StringImage (tmpbuf);
                         exit(1);
                     }
-
+#endif /* ! USE_DATABASE */
                 }
                 else
                 {
-                    if (CheckFile (filename) != 0)
+                    if (CheckFile (data_filename) != 0)
                         use_st=True;
                 }
 
@@ -763,10 +941,43 @@ char
                 ** read-write the counter datafile
                 */
 
-                dbuf=rwCounterDataFile(filename,
+#ifndef USE_DATABASE
+                dbuf=rwCounterDataFile(data_filename,
                         remote_ip,ignore_site,use_st,
                         digit_info.st,digit_info.increment_counter);
-    
+#else
+
+#ifdef SYS_WIN32
+                    /* make the full path of the database */
+                    (void) strcpy(db_filename,g_sZ_vbase);
+                    (void) strcat(db_filename,DATABASE_DIR);
+                    (void) strcat(db_filename,DIR_SEPARATOR);
+                    (void) strcat(db_filename,COUNT_DATABASE);
+#else
+                    /* make the full path of the database */
+                    (void) strcpy(db_filename,DATABASE_DIR);
+                    (void) strcat(db_filename,DIR_SEPARATOR);
+                    (void) strcat(db_filename,COUNT_DATABASE);
+#endif /* SYS_WIN32 */
+
+                /* if df param is provided, act the old way */
+                if (*digit_info.datafile != '\0')
+                {
+                    Debug2("XXX datafile=%s",data_filename,0);
+                    dbuf=rwCounterDataFile(data_filename,
+                        remote_ip,ignore_site,use_st,
+                        digit_info.st,digit_info.increment_counter);
+                }
+                else
+                {
+                    Debug2("db=%s",db_filename,0);
+                    dbuf=rwCounterDatabase(db_filename,
+                         digit_info.db_key,                 /* gg => new parameter */
+                         remote_ip,ignore_site,False,
+                         1,digit_info.increment_counter);
+                }
+#endif /* USE_DATABASE */
+
             }
     /* temporary things */
 
@@ -774,18 +985,18 @@ char
 
     if (dbuf != (char *) NULL)
     {
-        safeStrcpy(digitbuf,dbuf,sizeof(digitbuf)-1);
+        mutilsSafeStrcpy(digitbuf,dbuf,sizeof(digitbuf)-1);
         /* we don't need it anymore */
         (void) free(dbuf);
         dbuf=(char *) NULL;
     }
     else
     {
-        StringImage("unknown problem!");
+        StringImage("Failed to get counter numbers!");
         exit(0);
     }
 
-    counter_length = (int) strlen(digitbuf);
+        counter_length = (int) strlen(digitbuf);
 
         if (digit_info.show == False)
         {
@@ -837,17 +1048,13 @@ char
                 (void) sprintf(tmpbuf,"%s%s",tmpbuf,digitbuf);
                 digit_info.pdigits=mystrdup(tmpbuf);
                 makeCounterImage(&digit_info,&frame_info);
-                /*
-                ** we will not be here
-                */
+                /* we will not be here */
               }
               else    /* MaxDigits*/
               {
                   digit_info.pdigits=mystrdup(digitbuf);
                   makeCounterImage(&digit_info,&frame_info);
-                /*
-                ** we won't be here
-                */
+                /* we won't be here */
                }
 
             }
