@@ -5,10 +5,18 @@
 #include "combine.h"
 #include "count.h"
 
-static int  s_do_cache= 1;
-static int  s_user_reload=0;
-static int  s_expires=  300;
-static char *s_section= (char *) NULL;
+static int 
+    s_do_cache= 1;
+
+static int
+    s_user_reload=0;
+
+static int
+    s_expires=(-1);
+
+static char
+    *s_section= (char *) NULL,
+    *s_current_hits=(char *) NULL;
 
 /* set the value of s_do_cache to the passed value */
 /* no checking is done */
@@ -78,3 +86,20 @@ char *getcfgSection()
     return (s_section);
 }
 
+void setCurrentHits(hits)
+char
+    **hits;
+{
+    if (s_current_hits != (char *) NULL)
+    {
+        (void) free(s_current_hits);
+        s_current_hits=(char *) NULL;
+    }
+    if (*hits)
+        s_current_hits=mystrdup(*hits);
+}
+
+char *getCurrentHits()
+{
+    return(s_current_hits);
+}
