@@ -6,6 +6,13 @@
 #include "gdfonts.h"
 #include "count.h"
 
+#include "cdebug.h"
+
+#ifdef USE_DATABASE
+#include <ldbm.h>
+#endif /* USE_DATABASE */
+
+
 
 /*
  *  USE_DNS added by Sean Reifschneider <jafo@tummy.com> 1997-11-21.
@@ -32,15 +39,6 @@
 #include "config.h"
 #endif
 
-#ifdef __VMS
-#define DIR_SEPARATOR "" /* Already have ] at end of _DIR macros */
-/* VMS DIGIT_DIR ends in . so need to add subdirectory and ] - handled later */
-#define SYM_PREFIX "WWW_"
-#else
-#define DIR_SEPARATOR "/"
-#define SYM_PREFIX ""
-#endif
-
 static char *month_name[]=
 {
     "Jan", "Feb", "Mar", "Apr",
@@ -49,207 +47,8 @@ static char *month_name[]=
 };
 
 
-static int howmanyCommas _Declare((char *));
 static unsigned long my_inet_addr _Declare ((char *));
 
-/*
-**  howmanyCommas()
-**  calculates how many commas will be added to the buffer
-**
-**  RCS
-**      $Revision$
-**      $Date$
-**  Return Values:
-**      no of commas 
-**
-**  Parameters:
-**      buf     buf to scan
-**
-**  Side Effects:
-**      none
-**
-**  Limitations and Comments:
-**      buf must be a initialized string (at least)
-**
-**  Development History:
-**      who                  when           why
-**      ma_muquit@fccc.edu   Oct-18-1997    first cut
-*/
-static int howmanyCommas(buf)
-char
-    *buf;
-{
-    int
-        n=0,
-        length;
-
-    if (*buf == '\0')
-        return(0);
-
-    length=strlen(buf);
-
-    if ((length % 3) == 0)
-        n=(length/3)-1;
-    else
-        n=length/3;
-
-    if ( n < 0)
-        n=0;
-
-    Debug2("no of commas=%d",n,0);
-    return(n);
-}
-
-
-/*
-**  Commaize_buggy()
-**  add a comma after every 3rd digit from right
-**
-**  RCS
-**      $Revision$
-**      $Date$
-**  Return Values:
-**      none
-**
-**  Parameters:
-**      buf     buf to modify
-**
-**  Side Effects:
-**      buf is modified if it is atleast 4 character long
-**
-**  Limitations and Comments:
-************************************************************************
-**     ********** Don't use this, it's a bug!, mm, may-26-1999 *****
-************************************************************************
-**
-**  Development History:
-**      who                  when       why
-**      ma_muquit@fccc.edu   no idea    first cut
-**                           Oct-18-97  added dynamic buffer 
-*/
-
-
-void Commaize_buggy(buf)
-char
-    *buf;
-{
-    char
-        *pbuf=(char *) NULL;
-
-    register char
-        *p,
-        *q;
-
-    int
-        ncommas=0,
-        m=0,
-        i=-1;
-
-
-    if (*buf != '\0')
-    {
-        ncommas=howmanyCommas(buf);
-        /*
-        ** patch sent by James Burton <jamesb@aone.com.au>
-        ** Oct-21-1997
-        */
-        if (ncommas == 0)
-            return;
-
-        if (ncommas > 0) /* atleat 4 characters */
-        {
-            pbuf=(char *) malloc(strlen(buf)+ncommas*sizeof(char)+1);
-            if (pbuf == (char *) NULL)
-            {
-                /* return quietly */
-                return;
-            }
-        }
-        m=0;
-        m=m+strlen(buf)/3;
-        if (strlen(buf)%3 == 0)
-            m--;
-
-        p=buf+strlen(buf)-1;
-        pbuf[strlen(buf)+m]='\0';
-        q=pbuf+strlen(buf)+m-1;
-        while (1)
-        {
-            if (p < buf)
-                break;
-            if (++i == 3)
-                i=0, *q-- = ',';
-             *q-- = *p--;
-        }
-        (void) strcpy(buf,pbuf);
-        if (pbuf)
-            (void) free(pbuf);
-    }
-
-}
-
-
-/*
-**  Commaize()
-**  add a comma after every 3rd digit from right
-**
-**  RCS
-**      $Revision$
-**      $Date$
-**  Return Values:
-**      none
-**
-**  Parameters:
-**      buf     buf to modify
-**
-**  Side Effects:
-**      buf is modified
-**
-**  Limitations and Comments:
-**      buf must have enough space to hold the extra , characters
-**
-**  Development History:
-**      who                  when       why
-**      ma_muquit@fccc.edu   no idea    first cut
-**                           Oct-18-97  added dynamic buffer 
-**                           May-26-1999 rewrote as the old version has
-**                           memory overstepping bug. Code adapted from
-**                           C-Snippets, file commaflt.c by Bruce Wedding and
-**                           and Kurt Kuzba
-*/
-
-void Commaize(buf)
-char
-    *buf;
-{
-    char
-        *pbuf=(char *) NULL;
-
-    int
-        bf=0,
-        cm=0,
-        tm=0;
-
-    if (*buf != '\0')
-    {
-        pbuf=mystrdup(buf);
-        if (pbuf == (char *) NULL)
-            return; /* malloc failed, return quitely */
-
-        reverseString(pbuf);
-        while ((buf[bf++]=pbuf[tm++]) != 0)
-        {
-            if(++cm % 3 == 0 && pbuf[tm])
-                buf[bf++]=',';
-        }
-        if (pbuf)
-            (void) free((char *) pbuf);
-
-        reverseString(buf);
-    }
-}
-
-
 
 /*
 ** print the Content-type MIME header
@@ -263,12 +62,12 @@ void PrintHeader ()
 
     int
         exp;
-    /* 
-    ** check if caching should be disabled 
-    */
-    if (doHTTPcache() == 0)
+
+    exp=getExpireOffset();
+    Debug2("expires=%d",exp,0);
+
+    if (exp >= 0)
     {
-        exp=getExpireOffset();
         Debug2("Expire in: %d seconds",exp,0);
         getHTTPexpires(exp,expires,date,63,sizeof(expires)-1,sizeof(date)-1);
         Debug2("expires=%s",expires,0);
@@ -348,8 +147,8 @@ char
 **  a wrapper around malloc
 **
 **  RCS
-**      $Revision$
-**      $Date$
+**      $Revision: 1.5 $
+**      $Date: 2001/04/13 02:59:42 $
 **  Return Values:
 **      returns a pointer to allocated space
 **
@@ -386,210 +185,14 @@ int
 }
 
 
-/*
-** NULL terminate the buffer at the first sight of a non-digit character
-*/
-void cleanBuf(buf,bytes_in_buf,length)
-char
-    *buf;
-int
-    bytes_in_buf;
-int
-    *length;
-{
-    int
-        i;
-
-
-    int
-        c=0;
-    for (i=0; i < bytes_in_buf; i++)
-    {
-        if (!isdigit(buf[i]))
-        {
-            buf[i]='\0';
-            break;
-        }
-        c++;
-    }
-
-    *length=c;
-}
-/*
-**  safeStrcpy()
-**  copy a string to another safely without overflowing buffer
-**
-**  RCS
-**      $Revision$
-**      $Date$
-**  Return Values:
-**      none
-**
-**  Parameters:
-**      to_str      destination buffer
-**      from_str    source buffer
-**      length      max allowable length of the from_str
-**
-**  Side Effects:
-**      if copying is safe, to_str buffer is modified.
-**
-**  Limitations and Comments:
-**      if the source string is longer than length, it's truncated to
-**      length. to_str must be static or dynamically allocated buffer, that
-**      is it must be able to hold atleast lenth bytes buffer. The from_str
-**      is checked if it is NULL or not. if NULL the routine will exit 
-**      after writing an error message image.
-**      
-**
-**  Development History:
-**      who                  when           why
-**      ma_muquit@fccc.edu   Oct-17-1997    first cut
-*/
-
-void safeStrcpy(to_str,from_str,length)
-char
-    *to_str,
-    *from_str;
-int
-    length;
-{
-    /*
-    ** let turn off the followin message, I think we tested it enough :)
-    */
-    /*
-    if ( NULL == from_str )
-    {
-        StringImage("Source buffer is NULL in safeStrcpy()!");
-        exit(0);
-    }
-    */
-
-    /*
-    ** if source string is longer than to buffer, it will be 
-    ** truncated anyway, so let get rid of the buffer overflow message
-    */
-    /*
-    if (strlen(from_str) > length)
-    {
-        PrintHeader();
-        StringImage("buffer overflow detected! aborting");
-        exit(0);
-    }
-    */
-
-    /* now copy */
-    (void) strncpy(to_str,from_str,length);
-    to_str[length] = '\0';
-}
-
-/*
- *  safeStrcat()
- *  safely appends one string to another without overflowing the source
- *  buffer.
- *
- *  Parameters:
- *      to_str          appends with this string
- *      str             string to append
- *      length          first this many character of str to append
- *      ssc_size        maximum size of destination string
- *      ssc_length      length of string in destination buffer
- *
- *  Return Values:
- *      none
- *
- *  Limitations and Comments:
- *      to_str must have enough space staically or dynamically allocated
- *      before calling. Need to provide a portable strncat() as matt
- *      suggested that some versions of strncat (e.g. solaris 2.5.1) write
- *      at most n+1 characters past the end of s1 and then replace the 
- *      n+1st character with a '\0';
- *
- *
- *  Development History:
- *      who                  when           why
- *      ma_muquit@fccc.edu   Oct-1997       first cut
- *      mhpower@mit.edu      Nov-03-1997    fixed
- */
-
-
-void safeStrcat(to_str,str,length,ssc_size,ssc_length)
-char *to_str,
-     *str;
-int
-    length,
-    ssc_size, 
-    ssc_length;
-{
-   int copy_length;
-
-   if ( NULL == str )
-   {
-       StringImage("Source buffer is NULL in safeStrcat()!");
-       exit(0);
-   }
-
-   if (strlen(str) >= ssc_size - ssc_length)
-   {
-       StringImage("buffer overflow detected! aborting");
-       exit(0);
-   }
-
-   if (length < ssc_size - ssc_length)
-   {
-       copy_length = length;
-   }
-   else
-   {
-       copy_length = ssc_size - ssc_length;
-   }
-
-   /* now copy */
-   (void) mystrncat(to_str,str,copy_length);
-
-   /*
-   to_str[ssc_length + copy_length] = '\0';
-   */
-}
-
-
-
-/* case insensitive version of ANSI strstr() */
-/* from swish package by kevin h , kevin called it lstrstr()*/
-/*
-** s - hay stack
-** t - needle
-*/
-char *mystristr(s,t)
-char
-    *s;
-char
-    *t;
-{
-
-int
-    i,
-    j,
-    k,
-    l;
-
-    for (i = 0; s[i]; i++) 
-    {
-        for (j = 0, l = k = i; s[k] && t[j] &&
-            tolower(s[k]) == tolower(t[j]); j++, k++)
-            ;
-        if (t[j] == '\0')
-            return s + l;
-    }
-    return NULL;
-}
 
 /*
 **  authRemhost()
 **  authenticare remote host
 **
 **  RCS
-**      $Revision$
-**      $Date$
+**      $Revision: 1.5 $
+**      $Date: 2001/04/13 02:59:42 $
 **  Return Values:
 **      if HTTP_REFERER env variable exists:
 **        returns (1) only if host is authorized. othewise writes the error
@@ -641,7 +244,7 @@ int authRemhost()
 
     if (prem_refh != (char *) NULL)
     {
-        safeStrcpy(szHttp_ref,prem_refh,sizeof(szHttp_ref)-1);
+        mutilsSafeStrcpy(szHttp_ref,prem_refh,sizeof(szHttp_ref)-1);
         GetRemoteReferer(szHttp_ref,szRhost);
 
         if (*szRhost != '\0')
@@ -683,12 +286,9 @@ int authRemhost()
                     StringImage(szHttp_ref);
 
                     /*
-                    ** log the offending page too. Oops, the code was
-                    ** buggy! may be I was sleepy? Someone sent the fix.
-                    ** will put the name in ChangeLog.
-                    ** Sep-07-2000
+                    ** log the offending page too
                     */
-                    if (strlen(prem_refh) + 16 < sizeof(szbuf))
+                    if (strlen(prem_refh) < sizeof(szbuf)-1)
                     {
                         (void) sprintf(szbuf,"Referer page: %s\n",
                                        prem_refh);
@@ -707,135 +307,6 @@ int authRemhost()
 }
 
 
-/*
-**  isinname()
-**  checks if the mask fits in the string
-**
-**  RCS
-**      $Revision$
-**      $Date$
-**  Return Values:
-**      1       if fits
-**      0       if not
-**
-**  Parameters:
-**      string      source string
-**      mask        mask string
-**
-**  Example:
-**      www.fccc.edu  *.fccc.edu        fits
-**      www.fccc.edu  www*              fits
-**      www.fccc.edu  *fccc*            fits
-**      132.138.4.6   132*              fits
-**      etc.....
-**
-**  Side Effects:
-**      none
-**
-**  Limitations and Comments:
-**      borrowed from swish by Kevin H
-**
-**  Development History:
-**      who                  when           why
-**      ma_muquit@fccc.edu   Oct-18-1997    first cut
-*/
-
-int isinname(string, mask)
-char
-    *string;
-char
-    *mask;
-{
-    int
-        i,
-        j;
-
-    char
-        firstchar,
-        lastchar,
-        *tempmask;
-
-    if ((*string == '\0') || (*mask == '\0'))
-        return (0); /* mm */
-
-    if (!strcmp(mask, "*"))
-            return 1;
-
-    firstchar=mask[0];
-    lastchar=mask[(strlen(mask) - 1)];
-    tempmask=(char *) cMalloc(strlen(mask)+1);
-
-    for (i = j = 0; mask[i]; i++)
-        if (mask[i] != '*')
-            tempmask[j++] = mask[i];
-
-    tempmask[j]='\0';
-    if (firstchar == '*')
-    {
-        if (lastchar == '*')
-        {
-            if ((char *) mystristr(string, tempmask))
-            {
-                free(tempmask);
-                return 1;
-            }
-        }
-        else
-        {
-            if ((char *) mystristr(string, tempmask) ==
-                 string + strlen(string) - strlen(tempmask))
-            {
-                free(tempmask);
-                return 1;
-            }
-        }
-    }
-    else if (lastchar == '*')
-    {
-        if ((char *) mystristr(string, tempmask) == string)
-        {
-            free(tempmask);
-            return 1;
-        }
-    }
-    else
-    {
-        /*
-        ** changed from strcmp(), patch sent my
-        ** Takeshi OKURA <okura@osa.ncs.co.jp>
-        ** Oct-30-1997
-        */
-        if (!mystrcasecmp(string, tempmask))
-        {
-            free(tempmask);
-            return 1;
-        }
-    }
-    free(tempmask);
-
-    return 0;
-}
-
-
-
-/*
-** get current time
-*/
-
-char *GetTime ()
-{
-    time_t
-        tm;
-
-    char
-        *times;
-
-    tm = time (NULL);
-    times = ctime(&tm);
-    times[(int) strlen(times)-1] = '\0';
-    return (times);
-}
-
 void Warning (message)
 char
     *message;
@@ -856,12 +327,12 @@ char
     *buf='\0';
 
 #ifdef SYS_WIN32
-    (void) sprintf(buf,"%s%s/%s",g_sZ_vbase,LOG_DIR,LOG_FILE);
+    (void) sprintf(buf,"%s%s/%s",g_sZ_vbase,LOG_DIR,ERROR_LOG_FILE);
 #else
-    (void) sprintf(buf,"%s" DIR_SEPARATOR "%s",LOG_DIR,LOG_FILE);
+    (void) sprintf(buf,"%s%s%s",LOG_DIR,DIR_SEPARATOR,ERROR_LOG_FILE);
 #endif
 
-    times=GetTime();
+    times=mutilsGetTime();
 
 
     fp = fopen(buf, "a");
@@ -887,188 +358,123 @@ char
 }
 
 
-
 /*
- * Copyright (c) 1988, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-/*
-** I'm renaming it to mystrtok() in order to avoid conflict with the
-** system which might have it
-** I also formatted to my coding style
-** 10/08/95, muquit@semcor.com
+** Log visitor information
+** almost all comes from env variables
+** Log rotation code is taken from apache1.3.6 src/support/rotatelogs.c by
+** Ben Laurie <ben@algroup.co.uk>
+** -muquit, Aug-21-1999
 */
 
-char *mystrtok(s, delim)
-char
-    *s;
-char
-    *delim;
+void doVisitorLog()
 {
-	register char
-        *spanp;
+    FILE
+        *fp=NULL;
 
-	register int
-        c,
-        sc;
+    char
+        *vis_time,
+        *vis_host,
+        *vis_ip,
+        *vis_page,
+        *vis_current_hits,
+        *vis_browser,
+        tmpbuf[BUFSIZ],
+        szpath[MaxTextLength+1];
 
-	char
-        *tok;
-
-	static char
-        *last;
-
-
-	if (s == (char *) NULL && (s = last) == (char *) NULL)
-		return ((char *) NULL);
-
-	 /*
-	 ** Skip (span) leading delimiters (s += strspn(s, delim), sort of).
-	 */
-cont:
-	c = *s++;
-	for (spanp = (char *)delim; (sc = *spanp++) != 0;) 
+    time_t
+        tls,
+        trot;
+    
+    if (getcfgLogVisitorInfo() == True)
     {
-		if (c == sc)
-			goto cont;
-	}
 
-	if (c == 0) 
-    {		/* no non-delimiter characters */
-		last = (char *) NULL;
-		return ((char *)NULL);
-	}
-	tok = s - 1;
-
-	/*
-	 * Scan token (scan for delimiters: s += strcspn(s, delim), sort of).
-	 * Note that delim must have one NUL; we stop if we see that, too.
-	 */
-
-	for (;;) 
-    {
-		c = *s++;
-		spanp = (char *)delim;
-		do 
+        /* make the full path of the visitor log file */
+        /* check if we need to rotate logs, if so log file name will be */
+        /* Countx.xvisitor.log.time */
+#ifdef SYS_WIN32
+        (void) strcpy(szpath,g_sZ_vbase);
+        (void) strcat(szpath,LOG_DIR);
+        (void) strcat(szpath,DIR_SEPARATOR);
+        (void) strcat(szpath,VISITOR_LOG_FILE);
+#else
+        (void) strcpy(szpath,LOG_DIR);
+        (void) strcat(szpath,DIR_SEPARATOR);
+        (void) strcat(szpath,VISITOR_LOG_FILE);
+#endif /* SYS_WIN32 */
+        
+        trot=getcfgLogRotationInterval();
+        if (trot > 0)
         {
-			if ((sc = *spanp++) == c) 
-            {
-				if (c == 0)
-					s = (char *) NULL;
-				else
-					s[-1] = '\0';
-				last = s;
-				return (tok);
-			}
-		} while (sc != 0);
-	}
-	  /* NOTREACHED */
-}
+            /* log starts */
+            tls=(time(NULL)/trot)*trot;
 
-/*
-** muquit, first cut: Jan-21-1999
-** was ASI definition. fixed. apr-26-1999
-*/
-int mystrncasecmp(s1,s2,n)
-char
-    *s1,
-    *s2;
-int
-    n;
-{
-    register char
-        *scan1,
-        *scan2;
-
-    int
-        count;
-
-    scan1=s1;
-    scan2=s2;
-    count=n;
-
-    while (--count >= 0 && *scan1 != '\0' && tolower(*scan1) == tolower(*scan2))
-    {
-        scan1++;
-        scan2++;
-    }
-    if (count < 0)
-        return (0);
+#ifdef SYS_WIN32
+            (void) sprintf(szpath,"%s%s/%s_%010d",
+                           g_sZ_vbase,
+                           LOG_DIR,
+                           VISITOR_LOG_FILE,
+                           (int) tls);
 
 
-    return(tolower(*scan1) - tolower(*scan2));
-}
-
-
-/*
-** found somewhere in the net
-** muquit@semcor.com
-*/
-int mystrcasecmp(a,b)
-char
-    *a,
-    *b;
-{	
-    register char
-        ac,
-        bc;
-
-	for(;;)
-    {
-		ac = *a++;
-		bc = *b++;
-
-		if(ac == 0)
-        {
-			if(bc == 0)
-				return 0;
-			else
-                return -1;
+#else
+            (void) sprintf(szpath,"%s%s%s_%010d",
+                           LOG_DIR,
+                           DIR_SEPARATOR,
+                           VISITOR_LOG_FILE,
+                           (int) tls);
+#endif /* !SYS_WIN32 */
+            /* log ends */
+            /*
+            tle=tls+trot;
+            */
         }
-		else
-        { 
-            if(bc == 0)
-				return 1;
-			else 
-            {
-                if(ac != bc)
-                {
-					if(islower(ac)) ac = toupper(ac);
-					if(islower(bc)) bc = toupper(bc);
-					if( ac != bc )
-						return ac - bc;
-				}
-            }
-	    }
+
+        Debug2("visitor log file name=%s",szpath,0);
+        /* open file for writing */
+        fp=fopen(szpath,"a");
+        if (fp == (FILE *) NULL)
+        {
+            (void) sprintf(tmpbuf,"Could not open visitor log file:\n%s",
+                           szpath);
+            multilineStringImage(tmpbuf,ALIGN_CENTER,0,0,0,0,255,0);
+            exit(0);
+        }
+        vis_time=mutilsGetTime();
+        if (vis_time == NULL)
+            vis_time="<unknown>";
+
+        vis_host=getenv("REMOTE_HOST");
+        if (vis_host == NULL)
+            vis_host="<unknown>";
+
+        vis_ip=getenv("REMOTE_ADDR");
+        if (vis_ip == NULL)
+            vis_ip="<unknown>";
+
+        vis_page=getenv("HTTP_REFERER");
+        if (vis_page == NULL)
+            vis_page="<unknown>";
+        
+        vis_browser=getenv("HTTP_USER_AGENT");
+        if (vis_browser == NULL)
+            vis_browser="<unknown>";
+
+        vis_current_hits=getCurrentHits();
+        if (vis_current_hits == NULL)
+            vis_current_hits="<unknown>";
+
+        /*
+    (void) fprintf(fp, "Time: %s\nHost: %s\nIP: %s\nPage: %s\nCurrent Hits: %s\nBrowser:%s\n\n", vis_time,vis_host,vis_ip,vis_page,vis_current_hits,vis_browser);
+    */
+/*
+** format is:
+visit time|remote host|remote ip|page url|hit coutn|browser
+if any of the field is not found, it will be filled with <unknown>
+*/
+        (void) fprintf(fp,"%s|%s|%s|%s|%s|%s\n",
+            vis_time,vis_host,vis_ip,vis_page,vis_current_hits,vis_browser);
+
+        (void) fclose(fp);
     }
 }
 
@@ -1153,7 +559,10 @@ char *remote_ip, *ip_ign;
     else if (i == 1)  /* a single IP or wild card */
     {
         Debug2("in sing/wildcard ==> ignore IP=%s",ip_ign,0);
-        rc=isinname(remote_ip,ip_ign);
+        /*
+        rc=mutilsIsinname(remote_ip,ip_ign);
+        */
+        rc=mutilsWildmat(remote_ip,ip_ign);
         Debug2("ignore %s=%d",remote_ip,rc);
         return (rc);
     }
@@ -1187,8 +596,8 @@ char *s;
 **  get the today's date in GMT
 **
 **  RCS
-**      $Revision$
-**      $Date$
+**      $Revision: 1.5 $
+**      $Date: 2001/04/13 02:59:42 $
 **  Return Values:
 **      pointer to a static string buffer of asctime. 
 **
@@ -1235,8 +644,8 @@ char *getdateGMT()
 **  gets HTTP Expires and Date header
 **
 **  RCS
-**      $Revision$
-**      $Date$
+**      $Revision: 1.5 $
+**      $Date: 2001/04/13 02:59:42 $
 **  Return Values:
 **      none
 **
@@ -1301,8 +710,8 @@ int
     /*
     ** Date: header
     */
-    safeStrcpy(date,"Date: ",length);
-    safeStrcat(date,ptr,length,date_size,strlen(date));
+    mutilsSafeStrcpy(date,"Date: ",length);
+    mutilsSafeStrcat(date,ptr,length);
     (void) free(ptr);
 
     t_time_t += offset;
@@ -1314,8 +723,8 @@ int
     /*
     ** Expires: header
     */
-    safeStrcpy(expires,"Expires: ",length);
-    safeStrcat(expires,ptr,length,exp_size,strlen(expires));
+    mutilsSafeStrcpy(expires,"Expires: ",length);
+    mutilsSafeStrcat(expires,ptr,length);
     (void) free(ptr);
 }
 
@@ -1332,12 +741,8 @@ char
     if (string == (char *) NULL)
         return ((char *) NULL);
 
-    /*
-    **  malloc twice as much memory as a work around for something elsewhere 
-    **  muquit, may-26-1999
-    */
-    
-    tmp = (char *) malloc ((int) strlen(string)*2 );
+    /* malloc twice as much memory.  */
+    tmp = (char *) malloc ((int) strlen(string)*2);
 
     if (tmp == (char *) NULL)
     {
@@ -1374,85 +779,6 @@ char
             return;
     }
 }
-/*
- *  mystrncat()
- *  appends at most n characters of src string to dst string
- *
- *  Parameters:
- *      dst     destination string
- *      src     source string
- *      n       number of character to take from src
- *
- *  Return Values:
- *      pointer to dst string
- *
- *  Limitations and Comments:
- *      taken from Henry Spencer's public domain string library
- *
- *
- *  Development History:
- *      who                  when           why
- *      ma_muquit@fccc.edu   Nov-03-1997    first cut
- */
-
-char *mystrncat(dst,src,n)
-char
-    *dst,
-    *src;
-int
-    n;
-{
-    register char
-        *dscan,
-        *sscan;
-
-    register int
-        count;
-
-    for (dscan=dst; *dscan != '\0'; dscan++)
-        continue;
-    
-    sscan=src;
-    count=n;
-
-    while (*sscan != '\0' && --count >= 0)
-        *dscan++ = *sscan++;
-
-    *dscan++ = '\0';
-
-    return (dst);
-}
-
-/*
- * strncpy - copy at most n characters of string src to dst
- * adapted from henry's stringlib
- */
-char *              /* dst */
-mystrncpy(dst,src,n)
-char
-    *dst,
-    *src;
-int
-    n;
-{
-    register char
-        *dscan;
-    register char
-        *sscan;
-    register int
-        count;
-
-    dscan = dst;
-    sscan = src;
-    count = n;
-    while (--count >= 0 && (*dscan++ = *sscan++) != '\0')
-        continue;
-    while (--count >= 0)
-        *dscan++ = '\0';
-    return(dst);
-}
-
-
 
 
 /*
@@ -1561,8 +887,8 @@ time_t
  *  checkTimezone - parses the GMT+-hhmm and returns the time in seconds
  *
  *  RCS:
- *      $Revision: 2.3 $
- *      $Date: 1996/05/03 02:20:22 $
+ *      $Revision: 1.5 $
+ *      $Date: 2001/04/13 02:59:42 $
  *
  *  Security:
  *      Unclassified
@@ -1657,7 +983,8 @@ char
 **  getLocalTime()
 **  calculate time like: Mon-Day-Year hr:min:sec AM/PM GMT+xyzw
 **  Parameters:
-**  DigitInfo   *digitinfo  
+**  char *timezone_str      a string withe GMT offset, e.g. GMT+0500
+**                          GMT+0000 GMT-1030 etc
 **  int add_from            if 1, add From: before the time string
 **
 **  Return Values:
@@ -1672,21 +999,18 @@ char
 **  Development History:
 **      who                  when           why
 **      ma_muquit@fccc.edu   Jan-24-1999    first cut
-**                           Jul-17-1999    first arg is digit_info now
 */
 
-char  *getLocalTime(digit_info,add_from)
-DigitInfo
-    *digit_info;
+char  *getLocalTime(timezone_str,add_from)
+char
+    *timezone_str;
 int
     add_from;
 {
     char
-        *timezone_str,
         *s=(char *) NULL;
 
     struct tm
-        from_tm,
         *ptm;
 
     time_t
@@ -1695,31 +1019,7 @@ int
     long
         diff;
 
-    timezone_str=digit_info->time_z;
-
-    /* if from values specified, calculate time_t for them -- mm,jul17,99 */
-    if (digit_info->f_year >  0 &&
-        digit_info->f_mon  >  0 &&
-        digit_info->f_day  >  0 &&
-        digit_info->f_hr   >= 0 &&
-        digit_info->f_min  >= 0 &&
-        digit_info->f_sec  >= 0)
-    {
-        from_tm.tm_year=digit_info->f_year - 1900;
-        from_tm.tm_mon=digit_info->f_mon - 1;
-        from_tm.tm_mday=digit_info->f_day;
-        from_tm.tm_hour=digit_info->f_hr;
-        from_tm.tm_min=digit_info->f_min;
-        from_tm.tm_sec=digit_info->f_sec;
-        from_tm.tm_isdst=(-1);
-
-        now=mktime(&from_tm);
-    }
-    else
-    {
-        /* local time */
-        now=time(NULL);
-    }
+    now=time(NULL);
 
     if (timezone_str == NULL || *timezone_str == '\0')
     {
@@ -1782,48 +1082,256 @@ DigitInfo
     *digit_info;
 {
             /* make the gif filepath*/
-            safeStrcpy(path,base,path_len-1);
+            mutilsSafeStrcpy(path,base,path_len-1);
 #ifndef __VMS
-            safeStrcat(path,"/",path_len-1,path_len,strlen(path));
+            mutilsSafeStrcat(path,"/",path_len-1);
 #endif
 
-            safeStrcat(path,digit_info->ddhead,path_len-1,
-                    path_len,strlen(path));
+            mutilsSafeStrcat(path,digit_info->ddhead,path_len-1);
 #ifndef __VMS
-            safeStrcat(path,"/",path_len-1,path_len, strlen(path));
+            mutilsSafeStrcat(path,"/",path_len-1);
 #else
 /*          For VMS add a ] before adding name of .gif file  */
-            safeStrcat(path,"]",path_len-1,path_len,
-                    strlen(path));
+            mutilsSafeStrcat(path,"]",path_len-1);
 #endif
-            safeStrcat(path,digit_info->gif_file,path_len-1,
-                    path_len,strlen(path));
+            mutilsSafeStrcat(path,digit_info->gif_file,path_len-1);
 }
 
 
+
+
+char *mystrtok(s, delim)
+char
+    *s;
+char
+    *delim;
+{
+    register char
+        *spanp;
+
+    register int
+        c,
+        sc;
+
+    char
+        *tok;
+
+    static char
+        *last;
+
+
+    if (s == (char *) NULL && (s = last) == (char *) NULL)
+        return ((char *) NULL);
+
+     /*
+     ** Skip (span) leading delimiters (s += strspn(s, delim), sort of).
+     */
+cont:
+    c = *s++;
+    for (spanp = (char *)delim; (sc = *spanp++) != 0;)
+    {
+        if (c == sc)
+            goto cont;
+    }
+
+    if (c == 0)
+    {       /* no non-delimiter characters */
+        last = (char *) NULL;
+        return ((char *)NULL);
+    }
+    tok = s - 1;
+
+    /*
+     * Scan token (scan for delimiters: s += strcspn(s, delim), sort of).
+     * Note that delim must have one NUL; we stop if we see that, too.
+     */
+
+    for (;;)
+    {
+        c = *s++;
+        spanp = (char *)delim;
+        do
+        {
+            if ((sc = *spanp++) == c)
+            {
+                if (c == 0)
+                    s = (char *) NULL;
+                else
+                    s[-1] = '\0';
+                last = s;
+                return (tok);
+            }
+        } while (sc != 0);
+    }
+      /* NOTREACHED */
+}
+
+#ifdef USE_DATABASE
+
 /*
-** borrowed from c-snippets STRREV.C, public domain by Bob Stout
-** The name of the function was stttev()
-** -- muquit, May-26-1999
-** Comment: str is modified
+**  grepForStringInDb()
+**  check if a string is in the database
+**
+**  Parameters:
+**  char    *db_path        full path of the database
+**  char    *str            the string
+**  cahr    *errbuf         error message if it occurs
+**
+**  Return Values:
+**  True    if the string found
+**  False   if not
+**
+**  Limitations and Comments:
+**  the caller should check if any error message is in errbuf first.
+**  errbuf should have enough space to hold the message.
+**  search is case in-sensitive. 
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   Aug-13-1999    first cut
 */
-char *reverseString(str)
+
+
+int grepForStringInDb(db_path,str,errbuf)
+char
+    *db_path;
 char
     *str;
+char
+    *errbuf;
 {
-    char
-        *p1,
-        *p2;
+    LDBMCursor
+        *dbch=NULL;
 
-    if (! str || ! *str)
-        return str;
+    LDBM
+        dbp=NULL;
 
-    for (p1=str,p2 =str+strlen(str)-1; p2 > p1; ++p1, --p2)
+    Datum
+        key_data,
+        return_data;
+
+    int
+        rc=False;
+
+    if (*str == '\0' || str == NULL)
+        return (False);
+
+    ldbm_datum_init(key_data);
+    ldbm_datum_init(return_data);
+
+    /* open the database */
+    dbp=ldbm_open(db_path,LDBM_READER,00644,0);
+    if (dbp == NULL)
     {
-        *p1 ^= *p2;
-        *p2 ^= *p1;
-        *p1 ^= *p2;
+        (void) sprintf(errbuf,"Could not open database:\n%s",db_path);
+        return(False);
     }
-    return (str);
+
+    while (return_data.dptr != NULL)
+    {
+        key_data=return_data;
+        return_data=ldbm_fetch(dbp,key_data);
+        if (mutilsStristr(key_data.dptr,str)) /* found */
+        {
+            if (return_data.dptr)
+                ldbm_datum_free(dbp,return_data);
+            rc=True;
+            goto ExitProcessing;
+        }
+        if (return_data.dptr)
+            ldbm_datum_free(dbp,return_data);
+#if DB_VERSION_MAJOR >=2 || DB_VERSION_MAJOR >=3
+        return_data=ldbm_firstkey(dbp,&dbch);
+#else
+        return_data=ldbm_nextkey(dbp,key_data,dbch);
+#endif /* berkeley db < 2 */
+    }
+
+ExitProcessing:
+
+    if (dbp)
+        ldbm_close(dbp);
+
+    return (rc);
 }
 
+/*
+**  grepForKeyInDb()
+**  check if a key is in the database
+**
+**  Parameters:
+**  char    *db_path        full path of the database
+**  char    *str            the string
+**  cahr    *errbuf         error message if it occurs
+**
+**  Return Values:
+**      char *val       if the key is found
+**      NULL            if the key is not found or if value is NULL
+**
+**  Limitations and Comments:
+**  the caller should check if any error message is in errbuf first.
+**  errbuf should have enough space to hold the message. search is 
+**  case in-sensitive. 
+**  Returns pointer to a malloc'd space, the caller is responsible to
+**  free it.
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   Aug-13-1999    first cut
+**`                                         returns value if key is found
+*/
+
+
+char *grepForKeyInDb(db_path,key,errbuf)
+char
+    *db_path;
+char
+    *key;
+char
+    *errbuf;
+{
+    LDBM
+        dbp=NULL;
+
+    Datum
+        key_data,
+        return_data;
+
+    char
+        *val=NULL;
+
+    if (*key == '\0' || key == NULL)
+        return (NULL);
+
+    /* open the database */
+    dbp=ldbm_open(db_path,LDBM_READER,00644,0);
+    if (dbp == NULL)
+    {
+        (void) sprintf(errbuf,"Could not open database:\n%s",db_path);
+        return(NULL);
+    }
+
+    /*
+    ** initialize Datums. In Solaris 2.6 with gcc, it was not working
+    ** muquit@muquit.com Thu Apr 12 22:43:02 EDT 2001 
+    */
+    ldbm_datum_init(key_data);
+    ldbm_datum_init(return_data);
+
+    key_data.dptr=key;
+    key_data.dsize=strlen(key)+1;
+    return_data=ldbm_fetch(dbp,key_data);
+    if (return_data.dptr != NULL)
+    {
+        if (return_data.dptr)
+            val=mystrdup(return_data.dptr);
+        ldbm_datum_free(dbp,return_data);
+    }
+
+    if (dbp)
+        ldbm_close(dbp);
+
+    return (val);
+}
+
+#endif /* USE_DATABASE */
